"""
Estimate a process-based event probability
==========================================
"""

# %%
# The objective of this example is to evaluate the probability of an
# event based on a stochastic process, using the Monte Carlo estimator.
#
# Let :math:`X: \Omega \times \mathcal{D} \rightarrow \mathbb{R}^d` be a stochastic process
# of dimension :math:`d`, where :math:`\mathcal{D} \in \mathbb{R}^n` is discretized on the mesh
# :math:`\mathcal{M}`.
#
# We define the event :math:`\mathcal{E}` as:
#
# .. math::
#    \begin{aligned}
#      \displaystyle \mathcal{E}(X) = \bigcup_{\vect{t}\in \mathcal{M}}\left\{X_{\vect{t}}  \in \mathcal{A}  \right\}
#    \end{aligned}
#
# where :math:`\mathcal{A}` is a domain of :math:`\mathbb{R}^d`.
#
# We estimate the probabilty :math:`p=\mathbb{P}\left(\mathcal{E}(X)\right)` with the Monte Carlo
# estimator.
#
# The Monte Carlo algorithm is manipulated the same way as in the case
# where the event is based on a random variable independent of time.
#
# We illustrate the algorithm on the example of the bidimensionnal white
# noise process :math:`\varepsilon: \Omega \times \mathcal{D} \rightarrow \mathbb{R}^2` where
# :math:`\mathcal{D}\in \mathbb{R}`, distributed according to the bidimensionnal standard
# normal distribution (with zero mean, unit variance and independent
# marginals).
#
# We consider the domain :math:`\mathcal{A} =  [1,2] \times [1,2]`. Then the event :math:`\mathcal{E}` writes:
#
# .. math::
#    \begin{aligned}
#      \displaystyle \mathcal{E}(\varepsilon) = \bigcup_{\vect{t}\in \mathcal{M}}\left\{\varepsilon_{t}  \in \mathcal{A}  \right\}
#    \end{aligned}
#
# For all time stamps :math:`t \in \mathcal{M}`, the probability :math:`p_1` that the process
# enters into the domain :math:`\mathcal{A}` at time :math:`t` writes, using the independence
# property of the marginals:
#
# .. math::
#    \begin{aligned}
#      p_1 = \mathbb{P}\left(\varepsilon_t  \in \mathcal{A}\right) = (\Phi(2) - \Phi(1))^2
#    \end{aligned}
#
# with :math:`\Phi` the cumulative distribution function of the scalar standard *Normal* distribution.
#
# As the process is discretized on a time grid of size :math:`N` and using the
# independence property of the white noise between two different time
# stamps and the fact that the white noise follows the same distribution
# at each time :math:`t`, the final probability :math:`p` writes:
#
# .. math::
#    p = \mathbb{P}\left(\mathcal{E}(\varepsilon)\right) = 1 - (1 - p_1)^{N}
#
# With :math:`K=10^4` realizations, using the Monte Carlo estimator, we obtain :math:`p_K = 0.1627`,
# to be compared to the exact value :math:`p=0.17008` for a time grid of size :math:`N=10`.

# %%
import openturns as ot
import openturns.viewer as otv

# %%
# Create a time grid
tMin = 0.0
timeStep = 0.1
n = 100
tgrid = ot.RegularGrid(tMin, timeStep, n)

# %%
# Create a normal process
amplitude = [5.0]
scale = [3.0]
model = ot.ExponentialModel(scale, amplitude)
process = ot.GaussianProcess(model, tgrid)

# %%
# Create the 1-d domain A: [2.,5.]
lowerBound = [2.0]
upperBound = [5.0]
domain = ot.Interval(lowerBound, upperBound)

# %%
# Create an event from a Process and a Domain
event = ot.ProcessEvent(process, domain)

# %%
# Create the Monte-Carlo algorithm
montecarlo = ot.ProbabilitySimulationAlgorithm(event)

# Define the maximum number of simulations
montecarlo.setMaximumOuterSampling(1000)

# Define the block size
montecarlo.setBlockSize(100)

# Define the maximum coefficient of variation
montecarlo.setMaximumCoefficientOfVariation(0.0025)

# Run the algorithm
montecarlo.run()

# Get the result
montecarlo.getResult()

# %%
graph = montecarlo.drawProbabilityConvergence(0.95)
view = otv.View(graph)

# %%
otv.View.ShowAll()



#! /usr/bin/env python

import openturns as ot


# Default constructor: null polynomial
P0 = ot.MonomialFunction()
point = 1.2
print("P0 :", P0)
print("P0(", point, ")=", P0(point))


# Standard constructor : set degree/coefficients
degrees = range(1, 6)

for degree in degrees:
    print("\n")
    P = ot.MonomialFunction(degree)
    print("P :", P)
    print(
        "P(1) = %1.1f, dP(1) = %1.1f, d^2P(1) = %1.1f"
        % (P(1.0), P.gradient(1.0), P.hessian(1.0))
    )



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()


# TEST NUMBER ZERO : DEFAULT CONSTRUCTOR AND STRING CONVERTER
print("test number zero : default constructor and string converter")

# Default constructor
tensor0 = ot.Tensor()

# String converter
print("tensor0 = ", repr(tensor0))

# TEST NUMBER ONE : CONSTRUCTOR WITH SIZE, OPERATOR() AND STRING CONVERTER
print("test number one : constructor with size, operator() and string converter")

# Constructor with size
tensor1 = ot.Tensor(2, 2, 3)

# Check operator() methods
tensor1[0, 0, 0] = 1.0
tensor1[1, 0, 0] = 2.0
tensor1[0, 1, 0] = 3.0
tensor1[1, 1, 0] = 4.0
tensor1[0, 0, 1] = 5.0
tensor1[1, 0, 1] = 6.0
tensor1[0, 1, 1] = 7.0
tensor1[1, 1, 1] = 8.0
tensor1[0, 0, 2] = 9.0
tensor1[1, 0, 2] = 10.0
tensor1[0, 1, 2] = 11.0
tensor1[1, 1, 2] = 12.0

# String converter
print("tensor1 = ", repr(tensor1))
print(
    "values = %.6f" % tensor1[0, 0, 0],
    "  %.6f" % tensor1[1, 0, 0],
    "  %.6f" % tensor1[0, 1, 0],
    "  %.6f" % tensor1[1, 1, 0],
    "  %.6f" % tensor1[0, 0, 1],
    "  %.6f" % tensor1[1, 0, 1],
    "  %.6f" % tensor1[0, 1, 1],
    "  %.6f" % tensor1[1, 1, 1],
    "  %.6f" % tensor1[0, 0, 2],
    "  %.6f" % tensor1[1, 0, 2],
    "  %.6f" % tensor1[0, 1, 2],
    "  %.6f" % tensor1[1, 1, 2],
)

# TEST NUMBER TWO : COPY CONSTRUCTOR AND STRING CONVERTER
print("test number two : copy constructor and string converter")

# Copy constructor
tensor2 = tensor1

# String converter
print("tensor2 = ", repr(tensor2))

# TEST NUMBER THREE : GET DIMENSIONS METHODS
print("test number three : get dimensions methods")

# Get dimension methods
print("tensor1's nbRows = ", tensor1.getNbRows(), end=" ")
print("tensor1's nbColumns = ", tensor1.getNbColumns())
print("tensor1's nbSheets = ", tensor1.getNbSheets())

# TEST NUMBER FOUR : ASSIGNMENT METHOD
print("test number four : assignment method")

# Assignment method
# no sense with python

# TEST NUMBER FIVE : ISEMPTY METHOD
print("test number five : isEmpty method")

# Check method isEmpty
tensor5 = ot.Tensor()
tensor6 = ot.Tensor()
print("tensor1 is empty = ", tensor1.isEmpty())
print("tensor0 is empty = ", tensor0.isEmpty())
print("tensor5 is empty = ", tensor5.isEmpty())
print("tensor6 is empty = ", tensor6.isEmpty())

# TEST NUMBER SIX : GETSHEET AND SETSHEET METHODS
print("tensor1 = ", repr(tensor1))

#  Matrix sheet1(tensor1.getSheet(1))
print("tensor1.getSheet(1) = ", repr(tensor1.getSheet(1)))
sheet2 = ot.Matrix(2, 2)
sheet2[0, 0] = 0.5
sheet2[1, 0] = 0.6
sheet2[0, 1] = 0.7
sheet2[1, 1] = 0.8
print("sheet2 = ", repr(sheet2))
tensor1.setSheet(1, sheet2)
print("tensor1 = ", repr(tensor1))



import openturns as ot
import openturns.viewer as otv

size = 100
normal = ot.Normal(1)
sample = normal.getSample(size)

QQPlot = ot.VisualTest.DrawCDFplot(sample, normal)

otv.View(QQPlot, figure_kw={"figsize": (4.5, 4.5)})



r"""
An illustrated example of a FORM probability estimate
=====================================================
"""

# %%
# Abstract
# --------
#
# In this example we illustrate the different steps of a FORM/SORM analysis on a
# simple example. We focus on the different steps and compare them with an analytic
# computation whenever possible.
#
# See :ref:`FORM <form_approximation>` and :ref:`SORM <sorm_approximation>` and to get more theoretical details.
import openturns as ot
import openturns.viewer as otv
import numpy as np

# sphinx_gallery_thumbnail_number = 8

# %%
# Context
# -------
#
# We consider a bivariate random vector :math:`\inputRV = (X_1, X_2)` with the following independent components that follow:
#
# - the exponential distribution with parameter :math:`\lambda=1`, :math:`X_1 \sim \mathcal{E}(1.0)` ;
# - the standard unit normal distribution :math:`X_2 \sim \mathcal{N}(0,1)`.
#
# The support of the input vector is :math:`[0, +\infty[ \times \Rset`
dist_X1 = ot.Exponential(1.0)
dist_X2 = ot.Normal()
dist_X = ot.JointDistribution([dist_X1, dist_X2])


# %%
# We can draw the isolines of the PDF of the distribution `dist_X`:
ot.ResourceMap.SetAsUnsignedInteger("Contour-DefaultLevelsNumber", 8)
graph_PDF = dist_X.drawPDF([0.0, -10.0], [20.0, 10.0])
graph_PDF.setTitle(r"2D-PDF of the input variables $(X_1, X_2)$")
graph_PDF.setXTitle(r"$x_1$")
graph_PDF.setYTitle(r"$x_2$")
graph_PDF.setLegendPosition("lower right")
contours = graph_PDF.getDrawable(0).getImplementation()
contours.setColorMapNorm("rank")
contours.setIsFilled(True)
contours.buildDefaultLevels(50)
graph_PDF.setDrawable(0, contours)
view = otv.View(graph_PDF, square_axes=True)

# %%
# We consider the model from :math:`\Rset^2` into  :math:`\Rset` defined by:
#
# .. math::
#
#    \model : (x_1, x_2) \mapsto x_1 x_2
#
# We start by drawing the isolines of the model :math:`\model`.
g = ot.SymbolicFunction(["x1", "x2"], ["x1 * x2"])
graph_model = g.draw([0.0, -10.0], [20.0, 10.0])
graph_model.setXTitle(r"$x_1$")
graph_model.setYTitle(r"$x_2$")
graph_model.setTitle(r"Isolines of the model : $g$")
contours = graph_model.getDrawable(0).getImplementation()
contours.setColorMapNorm("rank")
contours.setIsFilled(True)
contours.buildDefaultLevels(50)
graph_model.setDrawable(0, contours)
view = otv.View(graph_model, square_axes=True)


# %%
# We consider  the univariate output variable :
#
# .. math::
#
#   Y = \model(\inputRV)
#
# We want to estimate the probability :math:`P_f` of the output variable to be greater than a
# prescribed threshold :math:`s=10` : this is the failure event.
# This probability is simply expressed for a continuous random vector :math:`\inputRV` as:
#
# .. math::
#    :label: PfDef
#
#    P_f = \Prob{Y \geq s} = \int_{\set{D}} \mathbf{1}_{\set{D}}(x) \pdf d\vect{x}
#
# where:
#
# .. math::
#
#  \set{D} = \{ (x_1, x_2) \in [0,+\infty[ \times \Rset \, | \,  \model(x_1, x_2) \geq s \}
#
# is the failure domain and :math:`\inputMeasure` is the probability density function (PDF)
# of :math:`\inputRV`.

# %%
# We first define random vectors
# and the failure event associated to the output random variable.
vector_X = ot.RandomVector(dist_X)
vector_Y = ot.CompositeRandomVector(g, vector_X)
s = 10.0
event = ot.ThresholdEvent(vector_Y, ot.Greater(), s)


# %%
# The boundary of the failure domain can easily be represented as it is a branch of an hyperbole: the
# boundary is the graph of the function defined from :math:`\Rset` into  :math:`\Rset` by:
#
# .. math::
#    :label: defH
#
#    h : x_1 \mapsto x_2 = \frac{s}{x_1}
#
# The boundary of the failure domain is also the isoline of the model :math:`\model` associated to the
# level :math:`s`:
#
# .. math::
#
#    \partial \set{D} =  \{(x_1, x_2)\, |\, \model(x_1, x_2) = s \}
#
# We can draw it with the `draw` method of the function :math:`\model`.

# %%
nb_points = 101
graph_g = g.draw([0.0, -10.0], [20.0, 10.0], [nb_points] * 2)
draw_boundary = graph_g.getDrawable(0)
draw_boundary.setLevels([s])
draw_boundary.setLegend(r"Boundary $\partial \mathcal{D}$")
graph_g.setDrawables([draw_boundary])

# %%
texts = [r" $\mathcal{D} = \{(x_1, x_2)\, |\, g(x_1, x_2) \geq 10 \}$"]
text_graph = ot.Text([[10.0, 3.0]], texts)
text_graph.setTextSize(1)
text_graph.setColor("black")
graph_g.add(text_graph)

# %%
graph_g.setTitle("Failure domain in the physical space")
graph_g.setXTitle(r"$x_1$")
graph_g.setYTitle(r"$x_2$")
graph_g.setLegendPosition("upper right")

view = otv.View(graph_g, square_axes=True)

# %%
# We can superimpose the event boundary with the bivariate PDF insolines of the input distribution:
draw_boundary.setColor("black")
graph_PDF.add(draw_boundary)
graph_PDF.setLegendPosition("lower right")
view = otv.View(graph_PDF, square_axes=True)

# %%
# From the previous figure, we observe that in the failure domain, the PDF takes small
# (and even very small) values.
# Consequently the failure probability :math:`P_f` is also expected to be small.
# The FORM/SORM methods estimate the failure probability.
#

# %%
# The FORM/SORM approximations
# ----------------------------
#
# The basic steps of the FORM and SORM algorithms are:
#
# - use an isoprobabilistic transformation to map the input random vector into the standard space;
# - find the design point which is the nearest point to the origin in the standard space;
# - estimate the probability.
#

# %%
# Isoprobabilistic transformation
# ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
#
# The interest of the  isoprobabilistic transformation is the rotational
# invariance of the distribution in the standard space. This property reduces the dimension
# of the problem to 1. See :ref:`Isoprobabilistic transformation <isoprobabilistic_transformation>` for more theoretical details.
#
# OpenTURNS has several isoprobabilistic transformations, depending on the distribution of the input random vector:
#
# - the Nataf transformation is used if the input distribution has a normal copula,
# - the Generalized Nataf transformation is used if the input distribution has an elliptical copula,
# - the Rosenblatt transformation is used in any other cases.
#
# The Nataf and  Rosenblatt transformations map the input random vector into a vector that follows a
# normal distribution with zero mean and unit variance. The Generalized Nataf transformation maps the
# input random vector into a vector that follows the standard elliptical distribution associated to the
# elliptical copula of the input distribution.
#
# In this example, the input distribution is not elliptical so the isoprobabilistic transformation is the
# Rosenblatt transformation.
#
print("Is Elliptical ? ", dist_X.isElliptical())

# %%
# The Rosenblatt transformation :math:`T` is defined by:
#
# .. math::
#    :label: defT
#
#    T : \vect{x} \mapsto \vect{z}
#
# such that the random vector :math:`\standardRV = T(\inputRV)` follows a bivariate normal distribution
# with zero mean and unit variance. It follows that the components :math:`Z_1` and
# :math:`Z_2` are independent.
#
# We detail the Rosenblatt transform in this simple case where the input random vector :math:`\inputRV`
# has independent components. Then, the Rosenblatt transform is defined by:
#
# .. math::
#
#    z_i = \Phi^{-1} \circ F_i(x_i)
#
# where :math:`F_i` is the cumulative distribution function (CDF) of :math:`X_i` and
# :math:`\Phi` the CDF of the univariate normal distribution with zero mean and unit variance.
# Note that in this example, :math:`\Phi^{-1} \circ F_2 = I_d` as :math:`F_2 = \Phi`.
# The isoprobabilistic transform and its inverse are methods of the distribution:
transformation = dist_X.getIsoProbabilisticTransformation()
inverse_transformation = dist_X.getInverseIsoProbabilisticTransformation()

# %%
# Let us detail this transformation, step by step.
# We draw a realization of the random input vector. This point is said to be in the physical space.
xi = vector_X.getRealization()

# %%
# We build `zi` the mapping of `xi` through the Rosenblatt transformation.
# The point `zi` is said to be in the standard space. Note that the second component remained unchanged.
ui = [dist_X1.computeCDF(xi[0]), dist_X2.computeCDF(xi[1])]
zi = [ot.Normal().computeQuantile(ui[0])[0], ot.Normal().computeQuantile(ui[1])[0]]
print(xi, "->", ui, "->", zi)

# %%
# We also build the isoprobabilistic transform :math:`T_1` and its inverse :math:`T_1^{-1}` for the
# first marginal:
#
# .. math::
#    :label: detT1
#
#    T_1 = \Phi^{-1} \circ F_1
#
transform_X1 = dist_X1.getIsoProbabilisticTransformation()
inverse_transform_X1 = dist_X1.getInverseIsoProbabilisticTransformation()

# %%
# We can implement the transformation using :math:`T_1` on the first components
# directly using :math:`T` on both components `xi`:
zi1D = [transform_X1([xi[0]])[0], xi[1]]
zi2D = transformation(xi)

# %%
# We can check the result of our experiment : we observe the results are the same.
print("zi = ", zi)
print("zi1D = ", zi1D)
print("zi2D = ", zi2D)


# %%
# The model in the standard space is defined by:
#
# .. math::
#
#     \widetilde{\model} = \model \circ T^{-1}
#
# We can define it using the capacities of the composition of functions implemented in the library.
g_tilde = ot.ComposedFunction(g, inverse_transformation)

# %%
# The failure domain in the standard space is defined by:
#
# .. math::
#
#  \set{\widetilde{D}} = \{ (z_1, z_2) \in [0,+\infty[ \times \Rset \, | \,  \widetilde{\model}(z_1, z_2) \geq s \}
#
# and its boundary is defined by:
#
# .. math::
#
#    \partial \set{\widetilde{D}} = \{ (z_1, z_2) \in [0,+\infty[ \times \Rset \, | \,
#       \widetilde{\model}(z_1, z_2) = s \}
#

# %%
# We draw the graph of :math:`\widetilde{g}` in the standard space.
graph_standard_space = g_tilde.draw([0.0, 0.0], [7.0, 7.0], [101] * 2)

draw_boundary_stand_space = graph_standard_space.getDrawable(0)
draw_boundary_stand_space.setLevels([s])
draw_boundary_stand_space.setLegend(r"Boundary $\partial \mathcal{\tilde{D}}$")
graph_standard_space.setDrawables([draw_boundary_stand_space])

graph_standard_space.setXTitle(r"$z_1$")
graph_standard_space.setYTitle(r"$z_2$")
graph_standard_space.setTitle("Failure domain in the standard space")

# %%
# We add some annotations
texts = [r"$\mathcal{\tilde{D}} = \{(z_1, z_2)\, |\, \tilde{g}(z_1, z_2) \geq 10 \}$"]
text_graph = ot.Text([[4.0, 3.0]], texts)
text_graph.setTextSize(1)
text_graph.setColor("black")
graph_standard_space.add(text_graph)

graph_standard_space.setLegendPosition("upper right")
view = otv.View(graph_standard_space, square_axes=True)

# %%
# The design point
# ^^^^^^^^^^^^^^^^
#
# Due to the spherical distribution in the standard space, the area that contributes
# the most to the integral defining the probability is the vicinity of the closest point
# of the failure domain to the origin of the standard space.
# Then the second step of the method is to find this point, *the design point*, through a
# minimization problem under constraints.

# %%
# We configure the Cobyla solver that we use for the optimization :
solver = ot.Cobyla()
solver.setMaximumIterationNumber(10000)
solver.setMaximumAbsoluteError(1.0e-3)
solver.setMaximumRelativeError(1.0e-3)
solver.setMaximumResidualError(1.0e-3)
solver.setMaximumConstraintError(1.0e-3)

# %%
# We build the FORM algorithm with its basic constructor. The starting point for the optimization
# algorithm is the mean of the input distribution.
solver.setStartingPoint(dist_X.getMean())
algo_FORM = ot.FORM(solver, event)

# %%
# We are ready to run the algorithm and store the result.
algo_FORM.run()
result = algo_FORM.getResult()

# %%
# The design point can be retrieved in both physical and standard space with respectively the
# `getPhysicalSpaceDesignPoint` and `getStandardSpaceDesignPoint` methods. We denote them respectively
# :math:`\vect{x}^*` and :math:`\vect{z}^*`.
design_point_physical_space = result.getPhysicalSpaceDesignPoint()
design_point_standard_space = result.getStandardSpaceDesignPoint()
print("Design point in physical space : ", design_point_physical_space)
print("Design point in standard space : ", design_point_standard_space)

# %%
# We can get the Hasofer index with the `getHasoferReliabilityIndex` method which is the distance of
# the design point to the origin:
beta_HL = result.getHasoferReliabilityIndex()
print("Hasofer index : ", beta_HL)

# %%
# We visualize the design point on the previous graph.
cloud = ot.Cloud([design_point_standard_space])
cloud.setColor("red")
cloud.setPointStyle("fcircle")
cloud.setLegend(r"design point $z^*$")
graph_standard_space.add(cloud)
graph_standard_space.setGrid(True)
graph_standard_space.setLegendPosition("lower right")
cc = ot.Curve(
    [0.0, design_point_standard_space[0]],
    [0.0, design_point_standard_space[1]],
    r"$\beta_{HL}$ distance",
)
cc.setLineStyle("dashed")
cc.setColor("black")
graph_standard_space.add(cc)
graph_standard_space.setLegendPosition("upper right")
view = otv.View(graph_standard_space, square_axes=True)

# %%
# The FORM approximation
# ^^^^^^^^^^^^^^^^^^^^^^
#
# The last step of the FORM algorithm is to replace the failure domain boundary by the hyperplane
# which is tangent to the failure domain at the design point in the standard space.
# To draw this hyperplane :math:`\mathcal{P}_{\vect{z}^*}`, we define the function from
# :math:`\Rset^2` to :math:`\Rset` defined by:
#
# .. math::
#
#    M \rightarrow \scalarproduct{\nabla \widetilde{\model}(\vect{z}^*)}{\vect{Z^*M}}
#
# where :math:`\nabla \vect{\widetilde{\model}(\vect{z}^*)}` is the gradient of the
# function :math:`\widetilde{\model}`
# at the design point :math:`Z^*(\vect{z}^*)`.
# Then, the tangent hyperplane is the isoline associated to the zero level of the previous function :
#
# .. math::
#
#    \mathcal{P}_{z^*} = \{ \vect{z} \in \Rset^2 \, | \,
#                \scalarproduct{\nabla\widetilde{\model}(\vect{z}^*)}{\vect{Z^*M}} = 0\}
#
# We can use the :class:`~openturns.LinearFunction` class.
center = design_point_standard_space
grad_design_point = g_tilde.gradient(design_point_standard_space)
constant = [0.0]
linear_mat = ot.Matrix(1, 2)
linear_mat[0, 0] = grad_design_point[0, 0]
linear_mat[0, 1] = grad_design_point[1, 0]
linear_proj = ot.LinearFunction(center, constant, linear_mat)

graph_tangent = linear_proj.getMarginal(0).draw([0.0, 0.0], [7.0, 7.0], [101] * 2)
draw_tangent = graph_tangent.getDrawable(0)
draw_tangent.setLevels([0])
draw_tangent.setLegend(r"$\mathcal{\Pi}_{z^*}$ (FORM)")
draw_tangent.setColor("green")
draw_tangent.setLineStyle("dashed")
graph_standard_space.add(draw_tangent)
graph_standard_space.setLegendPosition("upper right")
view = otv.View(graph_standard_space, square_axes=True)

# %%
# Depending on whether the origin of the standard space :math:`\vect{0}` belongs to the failure domain,
# the FORM probability is defined by:
#
# .. math::
#
#    P_{FORM} =  \begin{cases}
#                   E(+\beta_{HL}) & \text{ if }  \vect{0} \in \set{\widetilde{D}} \\
#                    E(-\beta_{HL}) & \text{ otherwise.}
#                 \end{cases}
#
# where :math:`E(.)` is the marginal cumulative distribution function along any direction of
# the spherical distribution in the standard space. In this example, this is the normal distribution.
# So we have:
#
isOriginFail = result.getIsStandardPointOriginInFailureSpace()
normal = ot.Normal()
if isOriginFail:
    pf_FORM = normal.computeCDF(beta_HL)
else:
    pf_FORM = normal.computeCDF(-beta_HL)
print("FORM : Pf_FORM = ", pf_FORM)

# %%
# This failure probability is implemented but the FORM algorithm and can be obtained
# with the `getEventProbability` method. We check we have the same result.
pf = result.getEventProbability()
print("Probability of failure (FORM) Pf_FORM  = ", pf)

# %%
# The SORM approximation
# ^^^^^^^^^^^^^^^^^^^^^^
#
# The SORM approximation uses the main curvatures :math:`\kappa_i^0` of the homothetic of the failure domain
# at distance 1 from the origin. These curvatures are calculated  at the design point.
# They are linked to the curvatures :math:`\kappa_i` of the failure domain by:
#
# .. math::
#
#    \kappa_i^0 = \beta_{HL} \kappa_i
#
# The Breitung approximation is valid for :math:`\beta_{HL} \rightarrow +\infty` and is defined by :
#
# .. math::
#
#    P_{SORM, Breitung} = \begin{cases}
#                 E(+\beta_{HL}) \prod_{i=1}^{d-1} \dfrac{1}{\sqrt{1+\kappa_i^0}} & \text{ if }
#                         \vect{0} \in \set{\widetilde{D}} \\
#                    E(-\beta_{HL}) \prod_{i=1}^{d-1} \dfrac{1}{\sqrt{1+\kappa_i^0}} & \text{ otherwise. }
#                 \end{cases}
#
# and approximates the boundary by the osculating paraboloid at the design point.
#
# Note that the term :math:`\kappa_i^0` does not depend on :math:`\beta_{HL}`.

# %%
# In this example, we can easily implement the boundary of the failure domain in the
# physical space, using the function :math:`h` defined in :eq:`defH`.
#
# In the standard space, the boundary is defined by the composed function
# :math:`z_1 \mapsto h \circ T_1^{-1}(z_1)`.
failure_boundary_physical_space = ot.SymbolicFunction(["x"], ["10.0 / x"])
failure_boundary_standard_space = ot.ComposedFunction(
    failure_boundary_physical_space, inverse_transform_X1
)
# %%
# We need the value of the second derivative of the failure boundary function
# at the abscissa of the design point in the standard space:
z1_star = [design_point_standard_space[0]]
dz1_star = failure_boundary_standard_space.getGradient().gradient(z1_star)
d2z1_star = failure_boundary_standard_space.getHessian().hessian(z1_star)
print("first component of the design point = ", z1_star[0])
print(
    "second component of the design point = ",
    failure_boundary_standard_space(z1_star)[0],
)
print(
    "value of the hessian of the failure boundary at this abscissa= ",
    d2z1_star[0, 0, 0],
)


# %%
# In the standard space, the osculating parabola :math:`\mathcal{P}_{\vect{z}^*}`
# at :math:`\vect{z}^*` is the graph of the function defined by:
#
# .. math::
#
#    z_1 \mapsto   h \circ T_1^{-1} (z_1^*) + \frac{d}{du_1} (h \circ T_1^{-1})(z_1^*) (z_1-z_1^*) +
#     \frac{1}{2} \frac{d^2}{dz_1^2} (h \circ T_1^{-1})(z_1^*) (z_1-z_1^*)^2
#
z = np.linspace(1.1, 4.0, 100)
parabola = (
    failure_boundary_standard_space(z1_star)[0]
    + dz1_star[0, 0] * (z - z1_star)
    + 0.5 * d2z1_star[0, 0, 0] * (z - z1_star) ** 2
)
curve_parabola = ot.Curve(z, parabola, r"$\mathcal{P}_{z^*}$ (SORM)")
curve_parabola.setLineStyle("dashed")
curve_parabola.setColor("orange")
graph_standard_space.add(curve_parabola)
graph_standard_space.setLegendPosition("upper right")
view = otv.View(graph_standard_space)


# %%
# The next step is to estimate the principal curvatures of the osculating paraboloid.
#
# For any regular function :math:`\ell: \Rset \rightarrow \Rset` the curvature :math:`\kappa(x)` at the point :math:`x` in
# cartesian coordinates reads as:
#
# .. math::
#
#    \kappa(x) = \frac{\ell''(x)}{(1+[\ell'(x)]^2)^{3/2}}.
#
# For the osculating parabola of concern we use the previously computed gradient and Hessian previously computed:
#
curvature = (d2z1_star[0, 0, 0]) / (1 + (dz1_star[0, 0]) ** 2) ** (3 / 2)
print("Curvature (analytic formula) = ", curvature)


# %%
# We build the SORM algorithm and run it:
solver.setStartingPoint(dist_X.getMean())
algo_SORM = ot.SORM(solver, event)
algo_SORM.run()

# %%
# The SORM result is obtained with the `getResult` method:
result_SORM = algo_SORM.getResult()

# %%
# The principal curvatures of the osculating paraboloid at the design point is obtained by the
# `getSortedCurvatures` method:
print("Curvature (library) = ", result_SORM.getSortedCurvatures()[1])


# %%
# Once the curvature is computed, there are several ways of approximating the failure probability :math:`P_f`.
# The library implements the Breitung, Hohenbichler and Tvedt estimates.
# We detail here the calculus of the Breitung approximation.
coeff = (1.0 + beta_HL * curvature) ** (-0.5)
if isOriginFail:
    pf_SORM = (normal.computeCDF(beta_HL)) * coeff
else:
    pf_SORM = (normal.computeCDF(-beta_HL)) * coeff
print("SORM : Pf_SORM = ", pf_SORM)

# %%
# We can compare with the different estimators:
pf_Breitung = result_SORM.getEventProbabilityBreitung()
pf_Hohenbichler = result_SORM.getEventProbabilityHohenbichler()
pf_Tvedt = result_SORM.getEventProbabilityTvedt()

print("Probability of failure (SORM Breintung) Pf = ", pf_Breitung)
print("Probability of failure (SORM Hohenbichler) Pf = ", pf_Hohenbichler)
print("Probability of failure (SORM Tvedt) Pf = ", pf_Tvedt)

# %%
# Display all figures
otv.View.ShowAll()



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()

data = ot.Point()

# DEBUG ; ici attente de la TUI
palette = ot.Description()
labels = ot.Description()

data.add(5665855)
labels.add("Candidate1")
palette.add("blue")

data.add(4804713)
labels.add("Candidate2")
palette.add("white")

data.add(4610113)
labels.add("Candidate3")
palette.add("pink")

data.add(1949170)
labels.add("Candidate4")
palette.add("yellow")

data.add(1630045)
labels.add("Candidate5")
palette.add("red")

data.add(1518528)
labels.add("Candidate6")
palette.add("ivory")

data.add(1495724)
labels.add("Candidate7")
palette.add("green")

data.add(1210562)
labels.add("Candidate8")
palette.add("orange")

data.add(1204689)
labels.add("Candidate9")
palette.add("maroon")

data.add(1113484)
labels.add("Candidate10")
palette.add("violet")

data.add(3295588)
labels.add("cumul candidates < 3.5% ")
palette.add("purple")

# Create an empty graph
myGraph = ot.Graph("Presidentielle 2002 - 1er tour", "x1", "x2")
myGraph.setLegendPosition("upper right")

# Create the first cloud
myPie1 = ot.Pie(data, labels, ot.Point(2), 1, palette)

# Then, draw it
myGraph.add(myPie1)



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()


# pNormal
xMin = 0.1
xMax = 0.9
nX = 10
grid = [0.0] * nX
for iX in range(nX):
    x = xMin + (xMax - xMin) * iX / (nX - 1)
    grid[iX] = x
    print(
        f"pNormal({x:.12g})={ot.DistFunc.pNormal(x):.6g}, complementary={ot.DistFunc.pNormal(x, True):.6g}"
    )
print("pNormal(", grid, "=", ot.DistFunc.pNormal(grid))
# qNormal
qMin = 0.0
qMax = 1.0
nQ = 11
grid = [0.0] * nQ
for iQ in range(nQ):
    q = qMin + (qMax - qMin) * iQ / (nQ - 1)
    grid[iQ] = q
    print(
        f"qNormal({q:.12g})={ot.DistFunc.qNormal(q):.6g}, complementary={ot.DistFunc.qNormal(q, True):.6g}"
    )
print("qNormal(", grid, ")=", ot.DistFunc.qNormal(grid))
# rNormal
nR = 10
for iR in range(nR):
    print(f"rNormal()={ot.DistFunc.rNormal():.6g}")



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()

distribution = ot.Dirichlet([0.2, 0.3, 0.4, 0.5])
size = 10000
sample = distribution.getSample(size)
factory = ot.DirichletFactory()
estimatedDistribution = factory.build(sample)
print("distribution=", repr(distribution))
print("Estimated distribution=", repr(estimatedDistribution))
estimatedDistribution = factory.build()
print("Default distribution=", estimatedDistribution)
estimatedDirichlet = factory.buildAsDirichlet(sample)
print("Dirichlet          =", distribution)
print("Estimated dirichlet=", estimatedDirichlet)
estimatedDirichlet = factory.buildAsDirichlet()
print("Default dirichlet=", estimatedDirichlet)



#! /usr/bin/env python

import openturns as ot

mesh = ot.RegularGrid(0.0, 0.1, 11)


class FUNC(ot.OpenTURNSPythonFieldFunction):
    def __init__(self):
        # first argument:
        super(FUNC, self).__init__(mesh, 2, mesh, 2)
        self.setInputDescription(["R", "S"])
        self.setOutputDescription(["T", "U"])

    def _exec(self, X):
        Xs = ot.Sample(X)
        Y = Xs * ([2.0] * Xs.getDimension())
        return Y

    def isActingPointwise(self):
        return True


F = FUNC()
print(
    "in_dim="
    + str(F.getInputDimension())
    + " out_dim="
    + str(F.getOutputDimension())
    + " spatial_dim="
    + str(F.getInputMesh().getDimension())
)


X = ot.Field(mesh, ot.Normal(2).getSample(11))
print(F(X.getValues()))

Xsample = ot.ProcessSample(5, X)
print(F(Xsample))


# Instance creation
myFunc = ot.FieldFunction(F)

# Copy constructor
newFunc = ot.FieldFunction(myFunc)

print(("myFunc input dimension= " + str(myFunc.getInputDimension())))
print(("myFunc output dimension= " + str(myFunc.getOutputDimension())))

print(myFunc(X.getValues()))

print(myFunc(Xsample))

print("point-wise?", myFunc.isActingPointwise())

vertices = []
vertices.append([0.0, 0.0, 0.0])
vertices.append([0.0, 0.0, 1.0])
vertices.append([0.0, 1.0, 0.0])
vertices.append([0.0, 1.0, 1.0])
vertices.append([1.0, 0.0, 0.0])
vertices.append([1.0, 0.0, 1.0])
vertices.append([1.0, 1.0, 0.0])
vertices.append([1.0, 1.0, 1.0])
simplicies = []
simplicies.append([0, 1, 2, 4])
simplicies.append([3, 5, 6, 7])
simplicies.append([1, 2, 3, 6])
simplicies.append([1, 2, 4, 6])
simplicies.append([1, 3, 5, 6])
simplicies.append([1, 4, 5, 6])
mesh3D = ot.Mesh(vertices, simplicies)


def myPyFunc(X):
    Xs = ot.Sample(X)
    Y = Xs * ([2.0] * Xs.getDimension())
    Y.setDescription(ot.Description.BuildDefault(values.getDimension(), "Y"))
    return Y


in_dim = 3
out_dim = 3

myFunc = ot.PythonFieldFunction(mesh3D, in_dim, mesh3D, out_dim, myPyFunc)
print("myFunc=", myFunc)

values = ot.Normal(mesh3D.getDimension()).getSample(mesh3D.getVerticesNumber())
# Evaluation over a single field
X = ot.Field(mesh3D, values)
print("X=", X)
Y = myFunc(X.getValues())
print("Y=", Y)
print("myFunc input dimension=", myFunc.getInputDimension())
print("myFunc output dimension=", myFunc.getOutputDimension())
print("myFunc input dimension=", myFunc.getInputMesh().getDimension())
print("called ", myFunc.getCallsNumber(), " times")
# Evaluation over a process sample
X = ot.ProcessSample(5, X)
print("X=", X)
Y = myFunc(X)
print("Y=", Y)
print("myFunc input dimension=", myFunc.getInputDimension())
print("myFunc output dimension=", myFunc.getOutputDimension())
print("myFunc input dimension=", myFunc.getInputMesh().getDimension())
print("called ", myFunc.getCallsNumber(), " times")



#!/usr/bin/env python

import openturns as ot
import openturns.testing as ott
import math as m


f = ot.SymbolicFunction(
    ["tau", "alpha"], ["cos(4*tau)*cosh((tau-pi_)/alpha)/cosh(pi_/alpha)"]
)
alpha = 1.36
rho = ot.ParametricFunction(f, [1], [alpha])

cov = ot.StationaryFunctionalCovarianceModel([1.0], [1.0], rho)
print(cov)

tau = 0.1
c = cov([tau])[0, 0]
print("tau=", tau)
print("c=", c)
c_ref = m.cos(4 * tau) * m.cosh((tau - m.pi) / alpha) / m.cosh(m.pi / alpha)
ott.assert_almost_equal(c, c_ref)

assert len(cov.getFullParameter()) == 4, "wrong full parameter"
assert len(cov.getFullParameterDescription()) == 4, "wrong full parameter description"

print(cov.getFullParameter())
print(cov.getFullParameterDescription())

assert len(cov.getActiveParameter()) == 2, "wrong active parameter"
cov.setActiveParameter(range(4))
cov.setParameter([1.0, 0.01, 1.0, 0.5])
assert len(cov.getParameter()) == 4, "wrong parameter"
c = cov([tau])[0, 0]
print("c=", c)
assert c != c_ref, "inactive parameter"

# test discretization
X = ot.Uniform(1, 2).getSample(10)
C = cov.discretize(X)
assert C.getNbRows() == 10, "wrong size"

# thread-safety test
for i in range(1000):

    def fun_mixte(X):
        xx, z = X
        if z == 0:
            y = m.sin(7.0 * xx)
        else:
            y = 2.0 * m.sin(7.0 * xx)
        return y

    XX_input = ot.Sample(
        [
            [0.1, 0],
            [0.32, 0],
            [0.6, 0],
            [0.9, 0],
            [0.07, 1],
            [0.1, 1],
            [0.4, 1],
            [0.5, 1],
            [0.85, 1],
        ]
    )
    y_output = ot.Sample(len(XX_input), 1)
    for i in range(len(XX_input)):
        y_output[i, 0] = fun_mixte(XX_input[i])

    def C(s, t):
        return m.exp(-4.0 * abs(s - t) / (1 + (s * s + t * t)))

    N = 32
    a = 4.0
    myMesh = ot.IntervalMesher([N]).build(ot.Interval(-a, a))

    myCovariance = ot.CovarianceMatrix(myMesh.getVerticesNumber())
    for k in range(myMesh.getVerticesNumber()):
        t = myMesh.getVertices()[k]
        for ll in range(k + 1):
            s = myMesh.getVertices()[ll]
            myCovariance[k, ll] = C(s[0], t[0])

    covModel_discrete = ot.UserDefinedCovarianceModel(myMesh, myCovariance)
    f_ = ot.SymbolicFunction(
        ["tau", "theta", "sigma"],
        [
            "(tau!=0) * exp(-1/theta) * sigma * sigma +  (tau==0) * exp(0) * sigma * sigma"
        ],
    )
    rho = ot.ParametricFunction(f_, [1, 2], [0.2, 0.3])
    covModel_discrete = ot.StationaryFunctionalCovarianceModel([1.0], [1.0], rho)
    covModel_continuous = ot.SquaredExponential([1.0], [1.0])
    covarianceModel = ot.ProductCovarianceModel(
        [covModel_continuous, covModel_discrete]
    )
    covarianceModel.discretize(XX_input)



#! /usr/bin/env python

import openturns as ot
import openturns.experimental as otexp
import openturns.testing as ott

ot.TESTPREAMBLE()

all_rules = [
    otexp.FejerExperiment.FEJERTYPE1,
    otexp.FejerExperiment.FEJERTYPE2,
    otexp.FejerExperiment.CLENSHAWCURTIS,
]

for rule in all_rules:
    for N in [1, 2, 3, 4, 5]:
        experiment = otexp.FejerExperiment([N], rule)
        experiment.setBounds(ot.Interval(4.0, 5.0))
        nodes, weights = experiment.generateWithWeights()
        print(f"---------- rule={rule} N={N} ----------")
        print(rule, nodes, weights)
        assert len(nodes) == N
        assert len(weights) == N
        assert experiment.hasUniformWeights() == (N < 2)
        ott.assert_almost_equal(nodes.computeMean(), [4.5])
        ott.assert_almost_equal(sum(weights), 2.0)

# check nesting
mapN = {otexp.FejerExperiment.FEJERTYPE2: lambda n: 2**n - 1}
mapN[otexp.FejerExperiment.CLENSHAWCURTIS] = lambda n: 2**n + 1
for rule in [otexp.FejerExperiment.FEJERTYPE2, otexp.FejerExperiment.CLENSHAWCURTIS]:
    prev_nodes = None
    for N in [1, 2, 3, 4, 5]:
        experiment = otexp.FejerExperiment([1], rule)
        experiment.setUseNestedLevels(True)
        experiment.setLevels([N])
        nodes, weights = experiment.generateWithWeights()
        print(f"---------- rule={rule} N={N} nested ----------")
        print(rule, nodes, weights)
        Nref = mapN[rule](N)
        assert len(nodes) == Nref
        assert len(weights) == len(nodes)
        # check previous nodes are nested in current nodes
        if prev_nodes is not None:
            tree = ot.KDTree(nodes)
            for x in prev_nodes:
                k = tree.query(x)
                d = ot.Point(x - nodes[k]).norm()
                assert d < 1e-15, f"x={x} xk={nodes[k]} d={d}"
        prev_nodes = nodes



#!/usr/bin/env python

import openturns as ot
from openturns.testing import assert_almost_equal

ot.RandomGenerator.SetSeed(0)
distribution_R = ot.LogNormalMuSigma(300.0, 30.0, 0.0).getDistribution()
distribution_F = ot.Normal(75e3, 5e3)
marginals = [distribution_R, distribution_F]
distribution = ot.JointDistribution(marginals)


model = ot.SymbolicFunction(["R", "F"], ["R - F / (pi_ * 100.0)"])

vect = ot.RandomVector(distribution)
g = ot.CompositeRandomVector(model, vect)
event = ot.ThresholdEvent(g, ot.Less(), -50.0)
algo = ot.StandardSpaceCrossEntropyImportanceSampling(event, 0.3)

algo.run()
result = algo.getResult()

assert_almost_equal(result.getProbabilityEstimate(), 0.000126895, 1.0e-2, 0.0)

# from intersection
paraboloid = ot.SymbolicFunction(
    ["u1", "u2", "u3", "u4", "u5"], ["- u5 + u1^2 + u2^2 + u3^2 + u4^2"]
)
b = 3.5
db = 0.02
dim = 5
U = ot.Normal(dim)
X = ot.RandomVector(U)
Y = ot.CompositeRandomVector(paraboloid, X)
E1 = ot.ThresholdEvent(Y, ot.Less(), -b)
E3 = ot.ThresholdEvent(Y, ot.Greater(), -(b + db))
E4 = ot.IntersectionEvent([E1, E3])
algo = ot.StandardSpaceCrossEntropyImportanceSampling(E4, 0.35)



import openturns as ot
import openturns.viewer as otv

ot.ResourceMap.SetAsUnsignedInteger("Contour-DefaultLevelsNumber", 10)

grid = ot.GridLayout(2, 2)
pdf_graph = ot.Graph("PDFs of 1D Dirichlet distributions", "x", "")
cdf_graph = ot.Graph("CDFs of 1D Dirichlet distributions", "x", "")
title_2d = "PDF of the Dirichlet dist. with theta = {}"
theta_1 = (2.0, 2.0, 2.0)
theta_2 = (0.5, 0.7, 0.9)
pdf_2d_1 = ot.Graph(title_2d.format(theta_1), "x1", "x2")
pdf_2d_2 = ot.Graph(title_2d.format(theta_2), "x1", "x2")
list_theta = [(2, 2), (7, 5), (2, 6), (3, 4)]
for theta in list_theta:
    distribution = ot.Dirichlet(theta)
    pdf_curve = distribution.drawPDF()
    cdf_curve = distribution.drawCDF()
    pdf_graph.add(pdf_curve)
    cdf_graph.add(cdf_curve)
legends = [f"theta={theta}" for theta in list_theta]
pdf_graph.setLegends(legends)
cdf_graph.setLegends(legends)
distribution_2d_1 = ot.Dirichlet(theta_1)
distribution_2d_2 = ot.Dirichlet(theta_2)
pdf_2d_1.add(distribution_2d_1.drawPDF())
pdf_2d_2.add(distribution_2d_2.drawPDF())
grid.setGraph(0, 0, pdf_graph)
grid.setGraph(0, 1, cdf_graph)
grid.setGraph(1, 0, pdf_2d_1)
grid.setGraph(1, 1, pdf_2d_2)
grid.setTitle("Dirichlet (theta)")
grid.setLegendPosition("upper right")
v = otv.View(grid)
fig = v.getFigure()
fig.axes[1].legend(loc="best")



"""
Use the ANCOVA indices
======================
"""

# %%
# In this example we are going to use the ANCOVA decomposition to estimate
# sensitivity indices from a model with correlated inputs.
#
# ANCOVA allows one to estimate the Sobol' indices, and thanks to a functional
# decomposition of the model it allows one to separate the part of variance
# explained by a variable itself from the part of variance explained by a
# correlation which is due to its correlation with the other input parameters.
#
#
# In theory, ANCOVA indices range is :math:`\left[0; 1\right]` ; the closer to 1 the index is,
# the greater the model response sensitivity to the variable is.
# These indices are a sum of a physical part :math:`S_i^U` and correlated part :math:`S_i^C`.
# The correlation has a weak influence on the contribution of :math:`X_i`, if :math:`|S_i^C|`
# is low and :math:`S_i` is close to :math:`S_i^U`.
# On the contrary, the correlation has a strong influence on the contribution of
# the input :math:`X_i`, if :math:`|S_i^C|` is high and :math:`S_i` is close to :math:`S_i^C`.
#
# The ANCOVA indices :math:`S_i` evaluate the importance of one variable at a time
# (:math:`d` indices stored, with :math:`d` the input dimension of the model).
# The :math:`d` uncorrelated parts of variance of the output due to each input :math:`S_i^U`
# and the effects of the correlation are represented by the indices resulting
# from the subtraction of these two previous lists.
#
# To evaluate the indices, the library needs of a functional chaos result
# approximating the model response with uncorrelated inputs and a sample with
# correlated inputs used to compute the real values of the output.
#

# %%
import openturns as ot
import openturns.viewer as otv

# %%
# Create the model (x1,x2) --> (y) = (4.*x1+5.*x2)
model = ot.SymbolicFunction(["x1", "x2"], ["4.*x1+5.*x2"])

# %%
# Create the input independent joint distribution
distribution = ot.Normal(2)
distribution.setDescription(["X1", "X2"])

# %%
# Create the correlated input distribution
S = ot.CorrelationMatrix(2)
S[1, 0] = 0.3
R = ot.NormalCopula.GetCorrelationFromSpearmanCorrelation(S)
copula = ot.NormalCopula(R)
distribution_corr = ot.JointDistribution([ot.Normal()] * 2, copula)

# %%
# ANCOVA needs a functional decomposition of the model
enumerateFunction = ot.LinearEnumerateFunction(2)
productBasis = ot.OrthogonalProductPolynomialFactory(
    [ot.HermiteFactory()] * 2, enumerateFunction
)
adaptiveStrategy = ot.FixedStrategy(
    productBasis, enumerateFunction.getStrataCumulatedCardinal(4)
)
samplingSize = 250
experiment = ot.MonteCarloExperiment(distribution, samplingSize)
X = experiment.generate()
Y = model(X)
algo = ot.FunctionalChaosAlgorithm(X, Y, distribution, adaptiveStrategy)
algo.run()
result = algo.getResult()

# %%
# Create the input sample taking account the correlation
size = 2000
sample = distribution_corr.getSample(size)

# %%
# Perform the decomposition
ancova = ot.ANCOVA(result, sample)
# Compute the ANCOVA indices (first order and uncorrelated indices are computed together)
indices = ancova.getIndices()
# Retrieve uncorrelated indices
uncorrelatedIndices = ancova.getUncorrelatedIndices()
# Retrieve correlated indices:
correlatedIndices = indices - uncorrelatedIndices

# %%
# Print Sobol' indices, the physical part, and the correlation part
print("ANCOVA indices ", indices)
print("ANCOVA uncorrelated indices ", uncorrelatedIndices)
print("ANCOVA correlated indices ", correlatedIndices)

# %%
graph = ot.SobolIndicesAlgorithm.DrawImportanceFactors(
    indices, distribution.getDescription(), "ANCOVA indices (Sobol')"
)
view = otv.View(graph)

# %%
graph = ot.SobolIndicesAlgorithm.DrawImportanceFactors(
    uncorrelatedIndices,
    distribution.getDescription(),
    "ANCOVA uncorrelated indices\n(part of physical variance in the model)",
)
view = otv.View(graph)

# %%
graph = ot.SobolIndicesAlgorithm.DrawImportanceFactors(
    correlatedIndices,
    distribution.getDescription(),
    "ANCOVA correlated indices\n(part of variance due to the correlation)",
)
view = otv.View(graph)

# %%
# Display all figures
otv.View.ShowAll()



#! /usr/bin/env python

import openturns as ot
import openturns.testing as ott

ot.TESTPREAMBLE()
ot.RandomGenerator.SetSeed(0)

print("Named colors=", ot.Drawable.GetValidColors())
hexa = ot.Drawable.ConvertFromRGB(31, 63, 127)
assert hexa == "#1f3f7f"
#
hexa = ot.Drawable.ConvertFromRGB(0.1, 0.2, 0.3)
assert hexa == "#1a334d"
#
hexa = ot.Drawable.ConvertFromRGBA(31, 63, 127, 191)
assert hexa == "#1f3f7fbf"
#
hexa = ot.Drawable.ConvertFromRGBA(0.1, 0.2, 0.3, 0.4)
assert hexa == "#1a334d66"
#
hexa = ot.Drawable.ConvertFromHSV(215.0, 0.2, 0.3)
assert hexa == "#3d444d"
#
hexa = ot.Drawable.ConvertFromHSVA(215.0, 0.2, 0.3, 0.4)
assert hexa == "#3d444d66"
#
rgb = ot.Drawable.ConvertFromHSVIntoRGB(215.0, 0.2, 0.3)
ott.assert_almost_equal(rgb, [0.24, 0.265, 0.3])
#
hsv = ot.Drawable.ConvertFromRGBIntoHSV(44.0 / 255.0, 160.0 / 255.0, 72.0 / 255.0)
ott.assert_almost_equal(hsv, (134.48275862068965, 0.725, 0.6274509803921569))
#
hsv = ot.Drawable.ConvertFromRGBIntoHSV(44.0 / 255.0, 72.0 / 255.0, 160.0 / 255.0)
ott.assert_almost_equal(hsv, (225.51724137931038, 0.725, 0.6274509803921569))
#
hsv = ot.Drawable.ConvertFromRGBIntoHSV(160.0 / 255.0, 44.0 / 255.0, 72.0 / 255.0)
ott.assert_almost_equal(hsv, (345.51724137931035, 0.725, 0.6274509803921569))
#
print("BuildDefaultPalette=", ot.Drawable.BuildDefaultPalette(10))
print("BuildRainbowPalette=", ot.Drawable.BuildRainbowPalette(10))
print("BuildTableauPalette=", ot.Drawable.BuildTableauPalette(10))
print("BuildDefaultPalette=", ot.Drawable.BuildDefaultPalette(20))
print("BuildTableauPalette=", ot.Drawable.BuildTableauPalette(20))
#
print("Line styles=", ot.Drawable.GetValidLineStyles())
print("Point styles=", ot.Drawable.GetValidPointStyles())
print("Fill styles=", ot.Drawable.GetValidFillStyles())
#
# Get/set the ResourceMap
name = ot.ResourceMap.GetAsString("Drawable-DefaultPaletteName")
assert name == "Tableau"
#
palette = ot.Drawable.BuildDefaultPalette(4)
assert palette == ["#1f77b4", "#ff7f0e", "#2ca02c", "#d62728"]
#
ot.ResourceMap.SetAsString("Drawable-DefaultPaletteName", "Rainbow")
palette = ot.Drawable.BuildDefaultPalette(4)
assert palette == ["#ff0000", "#ccff00", "#00ff66", "#0066ff"]
#
ot.ResourceMap.SetAsString("Drawable-DefaultPaletteName", "Tableau")
palette = ot.Drawable.BuildDefaultPalette(4)
assert palette == ["#1f77b4", "#ff7f0e", "#2ca02c", "#d62728"]
#
colors = ot.Drawable.ConvertValuesToColors(
    [1, 2.7, 4.5, 3.1, 5.2, 2.6], ["red", "blue", "green"], 1.0
)
assert colors == [
    "#ff0000ff",
    "#0000ffff",
    "#00ff00ff",
    "#0000ffff",
    "#00ff00ff",
    "#0000ffff",
]
colors = ot.Drawable.ConvertValuesToColors(
    [1, 2.7, 4.5, 3.1, 5.2, 2.6], ["red", "blue", "green"], 0.4
)
assert colors == [
    "#ff000066",
    "#0000ff66",
    "#00ff0066",
    "#0000ff66",
    "#00ff0066",
    "#0000ff66",
]



#!/usr/bin/env python

import openturns.testing as ott

# check assert_almost_equal
ott.assert_almost_equal(42.0, 42.0)
with ott.assert_raises(RuntimeError):
    ott.assert_almost_equal(42.0, 42.5)
ott.assert_almost_equal(0.0, 1e-12, 0.0, 1e-12)
with ott.assert_raises(RuntimeError):
    ott.assert_almost_equal(0.0, 1e-10, 0.0, 1e-12)
ott.assert_almost_equal(1.0, 1.0 + 1e-12, 1e-10, 0.0)
with ott.assert_raises(RuntimeError):
    ott.assert_almost_equal(1.0, 1e-10, 1e-12, 0.0)

ott.assert_almost_equal(float("inf"), float("inf"))
with ott.assert_raises(RuntimeError):
    ott.assert_almost_equal(float("-inf"), float("inf"))
with ott.assert_raises(RuntimeError):
    ott.assert_almost_equal(float("inf"), float("-inf"))
ott.assert_almost_equal(float("-inf"), float("-inf"))

ott.assert_almost_equal(float("nan"), float("nan"))
ott.assert_almost_equal(float("-nan"), float("nan"))
ott.assert_almost_equal(float("nan"), float("-nan"))
ott.assert_almost_equal(float("-nan"), float("-nan"))

ott.assert_almost_equal(complex(1.0, 2.0), complex(1.0, 2.0))
with ott.assert_raises(RuntimeError):
    ott.assert_almost_equal(complex(1.0, 2.0), complex(1.0, 3.0))
ott.assert_almost_equal(complex(1.0, float("inf")), complex(1.0, float("inf")))
ott.assert_almost_equal(complex(1.0, float("nan")), complex(1.0, float("nan")))
ott.assert_almost_equal(complex(1.0, float("inf")), complex(1.0, float("inf")))
ott.assert_almost_equal(complex(1.0, float("nan")), complex(1.0, float("nan")))
with ott.assert_raises(RuntimeError):
    ott.assert_almost_equal(complex(float("inf"), 0.0), complex(float("-inf"), 0.0))
with ott.assert_raises(RuntimeError):
    ott.assert_almost_equal(complex(1.0, float("-inf")), complex(1.0, float("inf")))



import openturns as ot
from math import pi

# Definition of the Ishigami function
dimension = 3
a = 7.0
b = 0.1
input_variables = ["xi1", "xi2", "xi3", "a", "b"]
formula = ["sin(xi1) + a * (sin(xi2)) ^ 2 + b * xi3^4 * sin(xi1)"]
full = ot.SymbolicFunction(input_variables, formula)
ishigami_model = ot.ParametricFunction(full, [3, 4], [a, b])

#  Generating a design of size
N = 150
# Considering independent Uniform distributions of dimension 3
# Bounds are (-pi,pi), (-pi,pi) and (-pi,pi)
distribution = ot.JointDistribution([ot.Uniform(-pi, pi)] * dimension)
bounds = distribution.getRange()
# Random LHS
lhs = ot.LHSExperiment(distribution, N)
lhs.setAlwaysShuffle(True)  # randomized
# Fixing C2 crit
space_filling = ot.SpaceFillingC2()
# Defining a temperature profile
temperatureProfile = ot.GeometricProfile()
# Pre conditioning : generate an optimal design with MC
nSimu = 100
algo = ot.MonteCarloLHS(lhs, nSimu, space_filling)
initialDesign = algo.generate()
result = algo.getResult()

print("initial design pre-computed. Performing SA optimization...")
# Use of initial design
algo = ot.SimulatedAnnealingLHS(
    initialDesign, distribution, space_filling, temperatureProfile
)
# Retrieve optimal design
input_database = algo.generate()

result = algo.getResult()

print("initial design computed")

# Response of the model
print("sampling size = ", N)
output_database = ishigami_model(input_database)

# Learning input/output
# Usual chaos meta model
enumerate_function = ot.HyperbolicAnisotropicEnumerateFunction(dimension)
orthogonal_basis = ot.OrthogonalProductPolynomialFactory(
    dimension * [ot.LegendreFactory()], enumerate_function
)
basis_size = 100
# Initial chaos algorithm
adaptive_strategy = ot.FixedStrategy(orthogonal_basis, basis_size)
# ProjectionStrategy ==> Sparse
fitting_algorithm = ot.KFold()
approximation_algorithm = ot.LeastSquaresMetaModelSelectionFactory(
    ot.LARS(), fitting_algorithm
)
projection_strategy = ot.LeastSquaresStrategy(approximation_algorithm)
print("Surrogate model...")
distribution_ishigami = ot.JointDistribution(dimension * [ot.Uniform(-pi, pi)])
algo_pc = ot.FunctionalChaosAlgorithm(
    input_database,
    output_database,
    distribution_ishigami,
    adaptive_strategy,
    projection_strategy,
)
algo_pc.run()
chaos_result = algo_pc.getResult()
print("Surrogate model computed")

# Validation
lhs_validation = ot.LHSExperiment(distribution_ishigami, 100)
input_validation = lhs_validation.generate()
output_validation = ishigami_model(input_validation)
# Chaos model evaluation
output_metamodel_sample = chaos_result.getMetaModel()(input_validation)

# Cloud validation ==> change from matplotlib to pure OT
# fig = plt.figure()
# plt.plot(output_validation, output_validation, 'b-', label='Model')
# plt.plot(output_validation, output_metamodel_sample, 'r.', label='SM')
# plt.title('Surrogate Model validation - Ishigami use case')
# plt.savefig('ishigami_model_validation.png')



#! /usr/bin/env python

import openturns as ot
import openturns.testing as ott

ot.TESTPREAMBLE()

# Instantiate one distribution object
x = [[1.0], [2.0], [3.0], [3.0]]
p = [0.3, 0.1, 0.6, 0.6]
distribution = ot.FiniteDiscreteDistribution(x, p)
print("Distribution ", repr(distribution))
print("Distribution ", distribution)

# Is this distribution elliptical ?
print("Elliptical = ", distribution.isElliptical())

# Is this distribution continuous ?
print("Continuous = ", distribution.isContinuous())

# Has this distribution an independent copula ?
print("Has independent copula = ", distribution.hasIndependentCopula())

# Test for realization of distribution
oneRealization = distribution.getRealization()
print("oneRealization=", repr(oneRealization))

# Test for sampling
size = 10
oneSample = distribution.getSample(size)
print("oneSample=Ok", repr(oneSample))

# Define a point
point = ot.Point(distribution.getDimension(), 2.0)

# Show PDF and CDF of a point
pointPDF = distribution.computePDF(point)
pointCDF = distribution.computeCDF(point)
print("point= ", repr(point), " pdf= %.12g" % pointPDF, " cdf=", pointCDF)

# Get 95% quantile
quantile = distribution.computeQuantile(0.95)
print("Quantile=", repr(quantile))
print("entropy=%.6f" % distribution.computeEntropy())

print("Standard representative=", distribution.getStandardRepresentative())
print("parameter=", distribution.getParameter())
print("parameterDescription=", distribution.getParameterDescription())
parameter = distribution.getParameter()
parameter[-1] = 0.3
distribution.setParameter(parameter)
print("parameter=", distribution.getParameter())

# To prevent automatic compaction
ot.ResourceMap.SetAsUnsignedInteger("FiniteDiscreteDistribution-SmallSize", 5)
sample = ot.Sample(40, 3)
for i in range(4):
    for j in range(3):
        sample[i, j] = 10 * (i // 3 + 1) + 0.1 * (j + 1)

multivariateUserDefined = ot.FiniteDiscreteDistribution(sample)
print("Multivariate FiniteDiscreteDistribution=", multivariateUserDefined)

# Has this distribution an independent copula ?
print("Has independent copula = ", multivariateUserDefined.hasIndependentCopula())

print("Marginal 0=", multivariateUserDefined.getMarginal(0))
print("Marginal (2, 0)=", multivariateUserDefined.getMarginal([2, 0]))

# cdf bug
loi_UD = ot.FiniteDiscreteDistribution(
    [[350], [358], [360], [353], [364], [355], [349], [351]]
)
assert loi_UD.computeCDF([349]) == 0.125, "wrong cdf at min"
assert loi_UD.computeCDF([364]) == 1.0, "wrong cdf at max"

ot.Log.Show(ot.Log.TRACE)
validation = ott.DistributionValidation(distribution)
validation.skipParameters()  # probabilities are renormalized so not independent
validation.run()

# alias
distribution = ot.UserDefined(x, p)



#! /usr/bin/env python

import openturns as ot
import openturns.testing as ott

ot.TESTPREAMBLE()

ot.RandomGenerator.SetSeed(0)

# First test: comparison with a Normal distribution with block-diagonal
# correlation
R0 = ot.CorrelationMatrix(2)
R0[0, 1] = 0.5
R1 = ot.CorrelationMatrix(3)
R1[0, 1] = 0.2
R1[0, 2] = 0.1
R1[1, 2] = 0.15
R2 = ot.CorrelationMatrix(2)
R2[0, 1] = 0.3
collection = [
    ot.Normal([0.0] * 2, [1.0] * 2, R0),
    ot.Normal([0.0] * 3, [1.0] * 3, R1),
    ot.Normal([0.0] * 2, [1.0] * 2, R2),
]
distribution = ot.BlockIndependentDistribution(collection)
copulaCollection = [ot.NormalCopula(R0), ot.NormalCopula(R1), ot.NormalCopula(R2)]
copula = ot.BlockIndependentCopula(copulaCollection)
ref = ot.JointDistribution([ot.Normal(0.0, 1.0)] * 7, copula)

# Define a point
point = [0.3] * distribution.getDimension()
print("Point= ", point)

# Show PDF and CDF of point
DDF = distribution.computeDDF(point)
print("ddf      =", DDF)
print("ddf (ref)=", ref.computeDDF(point))
PDF = distribution.computePDF(point)
print("pdf      =%.5f" % PDF)
print("pdf (ref)=%.5f" % ref.computePDF(point))
CDF = distribution.computeCDF(point)
print("cdf      =%.5f" % CDF)
print("cdf (ref)=%.5f" % ref.computeCDF(point))
Survival = distribution.computeSurvivalFunction(point)
print("Survival      =%.5f" % Survival)
print("Survival (ref)=%.5f" % ref.computeSurvivalFunction(point))
InverseSurvival = distribution.computeInverseSurvivalFunction(0.95)
print("Inverse survival      =", InverseSurvival)
print("Inverse survival (ref)=", ref.computeInverseSurvivalFunction(0.95))
print(
    "Survival(inverse survival)=%.5f"
    % distribution.computeSurvivalFunction(InverseSurvival)
)
# Get 50% quantile
quantile = distribution.computeQuantile(0.5)
print("Quantile      =", quantile)
print("Quantile (ref)=", ref.computeQuantile(0.5))
print("CDF(quantile) =%.5f" % distribution.computeCDF(quantile))

ot.Log.Show(ot.Log.TRACE)
validation = ott.DistributionValidation(distribution)
validation.setPDFTolerance(4e-3)  # for conditional PDF
validation.run()

# Instantiate one distribution object
R = ot.CorrelationMatrix(3)
R[0, 1] = 0.5
R[0, 2] = 0.25
collection = [
    ot.JointDistribution([ot.Normal()] * 2, ot.AliMikhailHaqCopula(0.5)),
    ot.Normal([1.0] * 3, [2.0] * 3, R),
    ot.JointDistribution([ot.Exponential()] * 2, ot.FrankCopula(0.5)),
]
distribution = ot.BlockIndependentDistribution(collection)
print("Distribution ", distribution)

# Is this distribution elliptical ?
print("Elliptical distribution= ", distribution.isElliptical())

# Is this distribution continuous ?
print("Continuous = ", distribution.isContinuous())

# Has this distribution an elliptical copula ?
print("Elliptical = ", distribution.hasEllipticalCopula())

# Has this distribution an independent copula ?
print("Independent = ", distribution.hasIndependentCopula())

# Test for realization of distribution
oneRealization = distribution.getRealization()
print("oneRealization=", oneRealization)

# Define a point
point = [0.3] * distribution.getDimension()
print("Point= ", point)

# Show PDF and CDF of point
DDF = distribution.computeDDF(point)
print("ddf     =", DDF)
PDF = distribution.computePDF(point)
print("pdf     =%.5f" % PDF)
CDF = distribution.computeCDF(point)
print("cdf=%.5f" % CDF)
Survival = distribution.computeSurvivalFunction(point)
print("Survival      =%.5f" % Survival)
print("Survival (ref)=%.5f" % distribution.computeSurvivalFunction(point))
InverseSurvival = distribution.computeInverseSurvivalFunction(0.95)
print("Inverse survival=", InverseSurvival)
print(
    "Survival(inverse survival)=%.5f"
    % distribution.computeSurvivalFunction(InverseSurvival)
)
# Get 50% quantile
quantile = distribution.computeQuantile(0.5)
print("Quantile=", quantile)
print("CDF(quantile)=%.5f" % distribution.computeCDF(quantile))

if distribution.getDimension() <= 2:
    # Confidence regions
    (
        interval,
        threshold,
    ) = distribution.computeMinimumVolumeIntervalWithMarginalProbability(0.95)
    print("Minimum volume interval=", interval)
    print("threshold=%.5f" % threshold)
    levelSet, beta = distribution.computeMinimumVolumeLevelSetWithThreshold(0.95)
    print("Minimum volume level set=", levelSet)
    print("beta=%.5f" % beta)
    (
        interval,
        beta,
    ) = distribution.computeBilateralConfidenceIntervalWithMarginalProbability(0.95)
    print("Bilateral confidence interval=", interval)
    print("beta=%.5f" % beta)
    (
        interval,
        beta,
    ) = distribution.computeUnilateralConfidenceIntervalWithMarginalProbability(
        0.95, False
    )
    print("Unilateral confidence interval (lower tail)=", interval)
    print("beta=%.5f" % beta)
    (
        interval,
        beta,
    ) = distribution.computeUnilateralConfidenceIntervalWithMarginalProbability(
        0.95, True
    )
    print("Unilateral confidence interval (upper tail)=", interval)
    print("beta=%.5f" % beta)

print("entropy     =%.5f" % distribution.computeEntropy())

mean = distribution.getMean()
# Ensure mean is [0,0,1,1,1,1,1]
# Following platform, the value slightly differs
ott.assert_almost_equal(distribution.getMean(), [0, 0, 1, 1, 1, 1, 1])

standardDeviation = distribution.getStandardDeviation()
print("standard deviation=", repr(standardDeviation))
skewness = distribution.getSkewness()
# print("skewness=", repr(skewness))
kurtosis = distribution.getKurtosis()
print("kurtosis=", repr(kurtosis))

dim = distribution.getDimension()
x = 0.6
y = [0.2] * (dim - 1)
print("conditional PDF=%.5f" % distribution.computeConditionalPDF(x, y))
print("conditional CDF=%.5f" % distribution.computeConditionalCDF(x, y))
print("conditional quantile=%.5f" % distribution.computeConditionalQuantile(x, y))
pt = ot.Point(dim)
for i in range(dim):
    pt[i] = 0.1 * i + 0.05
print("sequential conditional PDF=", distribution.computeSequentialConditionalPDF(pt))
resCDF = distribution.computeSequentialConditionalCDF(pt)
print("sequential conditional CDF(", pt, ")=", resCDF)
print(
    "sequential conditional quantile(",
    resCDF,
    ")=",
    distribution.computeSequentialConditionalQuantile(resCDF),
)

# Extract a 5-D marginal
dim = 5
point = [0.25] * dim
indices = [1, 2, 3, 5, 6]
print("indices=", indices)
margins = distribution.getMarginal(indices)
print("margins=", margins)
print("margins PDF=%.5f" % margins.computePDF(point))
print("margins CDF=%.5f" % margins.computeCDF(point))
quantile = margins.computeQuantile(0.95)
print("margins quantile=", quantile)
print("margins CDF(quantile)=%.5f" % margins.computeCDF(quantile))
print("margins realization=", margins.getRealization())
# Tests o the isoprobabilistic transformation
# General case with normal standard distribution
print(
    "isoprobabilistic transformation (general normal)=",
    distribution.getIsoProbabilisticTransformation(),
)
# General case with non-normal standard distribution
collection[0] = ot.SklarCopula(
    ot.Student(3.0, [1.0] * 2, [3.0] * 2, ot.CorrelationMatrix(2))
)
collection.append(ot.Triangular(2.0, 3.0, 4.0))
distribution = ot.BlockIndependentDistribution(collection)
print(
    "isoprobabilistic transformation (general non-normal)=",
    distribution.getIsoProbabilisticTransformation(),
)
dim = distribution.getDimension()
x = 2.6
y = [0.2] * (dim - 1)
q = 0.9
print("conditional PDF=%.5f" % distribution.computeConditionalPDF(x, y))
print("conditional CDF=%.5f" % distribution.computeConditionalCDF(x, y))
print("conditional quantile=%.5f" % distribution.computeConditionalQuantile(q, y))
pt = ot.Point(dim)
for i in range(dim):
    pt[i] = 0.1 * i + 0.05
print("sequential conditional PDF=", distribution.computeSequentialConditionalPDF(pt))
resCDF = distribution.computeSequentialConditionalCDF(pt)
print("sequential conditional CDF(", pt, ")=", resCDF)
print(
    "sequential conditional quantile(",
    resCDF,
    ")=",
    distribution.computeSequentialConditionalQuantile(resCDF),
)
print("range=", distribution.getRange())

# getStandardDeviation vs Dirac
distribution2 = ot.BlockIndependentDistribution([ot.Normal(), ot.Dirac(1800)])
ott.assert_almost_equal(distribution2.getStandardDeviation(), [1, 0])

# check marginal from a group is not uselessly wrapped in BlockIndependent
margins = distribution.getMarginal([0, 1])
ott.assert_almost_equal(margins, collection[0])

# check getSupport
distribution = ot.BlockIndependentDistribution(
    [ot.Multinomial(5, ot.Point(2, 0.25))] * 2
)
support = distribution.getSupport(ot.Interval([2] * 4, [5] * 4))
print(support)
assert support.getSize() == 9



#! /usr/bin/env python

import openturns as ot
import openturns.testing as ott

ot.TESTPREAMBLE()


# Default constructor: null polynomial
P = ot.UniVariatePolynomial()
point = 1.2
print("P=", P)
print("P(", point, ")=", P(point))

# Constructor: construct the 1D polynomial with a vector of Point
# e.g. UniVariatePolynomial P1 ( coefficients )
coefficients = ot.Point(6)
coefficients[0] = 1.3
coefficients[1] = 2.4
coefficients[2] = 2.0
coefficients[3] = -0.5
coefficients[4] = 4.0
coefficients[5] = 0.0

print("Initial coefficients=", coefficients)
P1 = ot.UniVariatePolynomial(coefficients)
print("P1=", repr(P1))
print("P1=", P1)
print("P1=", P1.__str__("Y", ""))
print("P1 degree=", P1.getDegree())
print("P1 roots=", P1.getRoots())
print("P1(", point, ")=", P1(point))

# Multiplication by a scalar (postfix only)
print("P1*2.0 =", P1 * 2.0)

# P1.incrementDegree(incDeg) : multiply P1 by X^incDeg
print("P1.incrementDegree(3)=", P1.incrementDegree(3))
print("P1.incrementDegree(0)=", P1.incrementDegree(0))
print("P1.incrementDegree(1)=", P1.incrementDegree(1))
print("P1.incrementDegree()=", P1.incrementDegree())

# Sum of two polynomials
coefficients = ot.Point(3)
coefficients[0] = -2.5
coefficients[1] = 3.5
coefficients[2] = -1.2
P2 = ot.UniVariatePolynomial(coefficients)
print("P2=", P2)
print("P1+P2=", P1 + P2)

# Subtraction of two polynomials
print("P1-P2=", P1 - P2)

# Multiplication of two polynomials
print("P1*P2=", P1 * P2)

# Multiplication of two polynomials using FFT
print("P1*P2=", P1 * P2)

# Create a polynomial with a leading term equal to 0 (do not print, then
# a first term with unity coefficient (hide the coefficient when printing),
# and then several coefficients equal to 1 or 0
P1 = ot.UniVariatePolynomial([0.0, 1.0, -2.0, -1.0, 0.0, 1.0])
print("P1=", P1)
print("(repr) P1=", repr(P1))
print("(html) P1=", P1._repr_html_())

# Evaluate a polynomial over a sample
x = [[1.0], [2.0], [3.0], [4.0], [5.0]]
y = P2(x)
yRef = [[-0.2], [-0.3], [-2.8], [-7.7], [-15.0]]
ott.assert_almost_equal(y, yRef)



import openturns as ot
from openturns.usecases import ishigami_function

uc = ishigami_function.IshigamiModel()
# add correlation
distribution = uc.distribution
R = ot.CorrelationMatrix(uc.dim)
R[1, 0] = 0.4
R[2, 0] = 0.25
R[2, 1] = 0.1
distribution.setCopula(ot.NormalCopula(R))
X = distribution.getSample(100)
Y = uc.model(X)
degree = 5
enumerateFunction = ot.LinearEnumerateFunction(uc.dim)
indexMax = enumerateFunction.getBasisSizeFromTotalDegree(degree)
polyColl = [0.0] * uc.dim
for i in range(uc.distribution.getDimension()):
    polyColl[i] = ot.StandardDistributionPolynomialFactory(
        uc.distribution.getMarginal(i)
    )
productBasis = ot.OrthogonalProductPolynomialFactory(polyColl, enumerateFunction)
r2 = {}
for useDomination in [False, True]:
    algo = ot.LeastSquaresExpansion(X, Y, distribution, productBasis, indexMax)
    algo.setUseDomination(useDomination)
    algo.run()
    pce_result = algo.getResult()
    assert pce_result.getUseDomination() == useDomination
    surrogate = pce_result.getMetaModel()
    yPred = surrogate(X)
    validation = ot.MetaModelValidation(Y, yPred)
    r2Score = validation.computeR2Score()[0]
    print(f"{useDomination=} {r2Score=:.6f}")
    r2[useDomination] = r2Score
assert r2[True] > r2[False]

# hot start test
f = ot.SymbolicFunction(["x", "y"], ["sin(x+y)"])
d = ot.ClaytonCopula(1.5)
size = 1000
inL = d.getSample(size)
outL = f(inL)
inT = d.getSample(size)
outT = f(inT)
#
algo = ot.LeastSquaresExpansion(inL, outL, d)
algo.setUseDomination(False)
algo.run()
validation = ot.MetaModelValidation(algo.getResult().getMetaModel()(inT), outT)
print(f"No domination, R2={validation.computeR2Score()[0]}")
algo.setUseDomination(True)
algo.run()
validation = ot.MetaModelValidation(algo.getResult().getMetaModel()(inT), outT)
r2_hot = validation.computeR2Score()[0]
print(f"Domination after first run, R2={r2_hot}")
algo = ot.LeastSquaresExpansion(inL, outL, d)
algo.setUseDomination(True)
algo.run()
validation = ot.MetaModelValidation(algo.getResult().getMetaModel()(inT), outT)
r2_cold = validation.computeR2Score()[0]
print(f"Domination from scratch, R2={r2_cold}")
assert r2_hot == r2_cold



"""
Estimate tail dependence coefficients on the wind data
======================================================
"""

# %%
# In this example we estimate the tail dependence coefficient of a bivariate sample
# applied to the corresponding annual maximum wind speeds over the period 1944-1983
# at two locations in the United States: Albany, New-York and Hartford, Connecticut.
# Readers should refer to [coles2001]_ to get more details.
#
# First, we load the wave_surge dataset. The speeds are expressed in knot : one knot is equalt to one nautical mile per hour, which means :math:`1.852\, km.h^{-1}`.
import openturns as ot
import openturns.viewer as otv
from openturns.usecases import coles

data = coles.Coles().wind[:, 1:]
print(data[:5])

graph = ot.Graph(
    "Annual maximum wind speeds at Albany (NY) and Hartford (CT)",
    "spped at Albany (knot)",
    "speed at Hartford (knot)",
    True,
    "",
)
cloud = ot.Cloud(data)
cloud.setColor("red")
graph.add(cloud)
view = otv.View(graph)

# %%
# We plot the graph of the function :math:`u \mapsto \chi(u)` and the graph of the function
# :math:`u \mapsto \bar{\chi}(u)`. We conclude that both variables are asymptotially dependent
# as :math:`\chi > 0` and that they are positively correlated as :math:`\hat{\chi} > 0`.
# We cn visually deduce the upper tail dependence coefficient :math:`\chi \simeq 0.26` and
# the upper extremal dependence coefficient :math:`\bar{\chi} \simeq 0.5`.
# Note that the number of data points is so small that the curves seem chaotic.
# It is difficult, if not impossible, to deduce the value of :math:`\chi` and :math:`\bar{\chi}` from the curves.
graph1 = ot.VisualTest.DrawUpperTailDependenceFunction(data)
graph2 = ot.VisualTest.DrawUpperExtremalDependenceFunction(data)
grid = ot.GridLayout(1, 2)
grid.setGraph(0, 0, graph1)
grid.setGraph(0, 1, graph2)
view = otv.View(grid)

# %%
otv.View.ShowAll()



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()

mesh = ot.RegularGrid(0.0, 0.1, 11)


class FUNC(ot.OpenTURNSPythonPointToFieldFunction):
    def __init__(self):
        super(FUNC, self).__init__(2, mesh, 2)
        self.setInputDescription(["R", "S"])
        self.setOutputDescription(["T", "U"])
        self.mesh_ = ot.RegularGrid(0.0, 0.1, 11)

    def _exec(self, X):
        size = self.mesh_.getVerticesNumber()
        values = [ot.Point(X) * i for i in range(size)]
        Y = ot.Field(self.mesh_, values)
        return Y


F = FUNC()
print(
    "in_dim=" + str(F.getInputDimension()) + " out_dim=" + str(F.getOutputDimension())
)

print((F((10.0, 5.0))))

print((F(((10.0, 5.0), (6.0, 7.0)))))


# Instance creation
myFunc = ot.PointToFieldFunction(F)

# Copy constructor
newFunc = ot.PointToFieldFunction(myFunc)

print(("myFunc input dimension= " + str(myFunc.getInputDimension())))
print(("myFunc output dimension= " + str(myFunc.getOutputDimension())))

inPt = ot.Point(2, 2.0)
print((repr(inPt)))

outPt = myFunc(inPt)
print((repr(outPt)))

outPt = myFunc((10.0, 11.0))
print((repr(outPt)))

inSample = ot.Sample(10, 2)
for i in range(10):
    inSample[i] = ot.Point((i, i))
print((repr(inSample)))

outSample = myFunc(inSample)
print((repr(outSample)))

outSample = myFunc(((100.0, 100.0), (101.0, 101.0), (102.0, 102.0)))
print((repr(outSample)))



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()

# Dimension of the input model
# Size of the TimeGrid
# dimension parameter
dimension = 1

# Amplitude values
amplitude = ot.Point(dimension, 1.00)

# Scale values
scale = ot.Point(dimension, 1.0)

size = 128
timeGrid = ot.RegularGrid(0.0, 0.1, size)

# Cauchy model
model = ot.CauchyModel(scale, amplitude)
covModel = ot.AbsoluteExponential(scale, amplitude)
myProcess = ot.SpectralGaussianProcess(model, timeGrid)

# Create a sample of size N = 1000
sample = myProcess.getSample(1000)

# StationaryCovarianceModelFactory using default parameter - Factory
# initiate
myFactory = ot.StationaryCovarianceModelFactory()

# Build a UserDefinedCovarianceModel using the Wellch method
myCovarianceModel = myFactory.buildAsUserDefinedStationaryCovarianceModel(sample)
tg = myCovarianceModel.getTimeGrid()

# Get the time grid of the model
for i in range(tg.getN()):
    t = tg.getValue(i)
    estimatedValue = myCovarianceModel(t)[0, 0]
    modelValue = covModel(t)[0, 0]
    print(
        "Covariance C( %.6g" % t,
        ") : ",
        " evaluation =  %.6g" % estimatedValue,
        " model =  %.6g" % modelValue,
    )



#! /usr/bin/env python

import openturns as ot

# We create an empty Matrix
N = 10
d = 5

matrices = [ot.Matrix(N, d), ot.ComplexMatrix(N, d)]
for matrix in matrices:
    # We populate the empty sample with numerous data
    for i in range(matrix.getNbRows()):
        for j in range(matrix.getNbColumns()):
            matrix[i, j] = 100.0 * (j + 1) + i
    print("matrix=", str(matrix))

    # We try to slice the sample

    # Reading slicing

    # case #0 [slice]
    print("rslice[:]=", matrix[:])
    print("rslice[2:4]=", matrix[2:4])
    print("rslice[1:8:2]=", matrix[1:8:2])

    # case #1 [slice/slice]
    print("rslice[:,:]=", matrix[:, :])
    print("rslice[2:4,2:4]=", matrix[2:4, 2:4])
    print("rslice[1:8:2,1:5:3]=", matrix[1:8:2, 1:5:3])

    # case #2 [slice/index]
    print("rslice[:,3]=", matrix[:, 3])
    print("rslice[2:7:2,3]=", matrix[2:7:2, 3])

    # case #3 [index/slice]
    print("rslice[3,:]=", matrix[3, :])
    print("rslice[3,1:4:2]=", matrix[3, 1:4:2])

    # case #4 [index/index]
    print("rslice[3,4]=", matrix[3, 4])

    # Writing slice

    # case #0 [slice]
    matrix2 = matrix
    matrix2[:] = matrix[:]
    print("wrslice[:]=", matrix2)
    matrix2 = matrix
    matrix2[2:4] = matrix[2:4]
    print("wslice[2:4]=", matrix2)
    matrix2 = matrix
    matrix2[1:8:2] = matrix[1:8:2]
    print("wslice[1:8:2]=", matrix2)

    # case #1 [slice/slice]
    matrix2 = matrix
    matrix2[:, :] = matrix[:, :]
    print("wrslice[:,:]=", matrix2)
    matrix2 = matrix
    matrix2[2:4, 2:4] = matrix[2:4, 2:4]
    print("wslice[2:4,2:4]=", matrix2)
    matrix2 = matrix
    matrix2[1:8:2, 1:5:3] = matrix[1:8:2, 1:5:3]
    print("wslice[1:8:2,1:5:3]=", matrix2)

    # case #2 [slice/index]
    matrix2 = matrix
    matrix2[:, 3] = matrix[:, 3]
    print("wslice[:,3]=", matrix2)
    matrix2 = matrix
    matrix2[2:7:2, 3] = matrix[2:7:2, 3]
    print("wslice[2:7:2,3]=", matrix2)

    # case #3 [index/slice]
    matrix2 = matrix
    matrix2[3, :] = matrix[3, :]
    print("wslice[3,:]=", matrix2)
    matrix2 = matrix
    matrix2[3, 1:4:2] = matrix[3, 1:4:2]
    print("wslice[3,1:4:2]=", matrix2)

    # case #4 [index/index]
    matrix2 = matrix
    matrix2[3, 4] = matrix[3, 4]
    print("wslice[3,4]=", matrix2)



"""
Create a deterministic design of experiments
============================================
"""

# %%
import openturns as ot
import openturns.viewer as otv

# %%
# Four types of deterministic designs of experiments are available:
#
# - :class:`~openturns.Axial`
# - :class:`~openturns.Factorial`
# - :class:`~openturns.Composite`
# - :class:`~openturns.Box`
#
# Each type of deterministic design is discretized differently according to a number of levels.
#
# Functionally speaking, a design is a :class:`~openturns.Sample` that lies within the unit cube :math:`(0,1)^d` and can be scaled and moved to cover the desired box.


# %%
# We will use the following function to plot bi-dimensional samples.


# %%
def drawBidimensionalSample(sample, title):
    n = sample.getSize()
    graph = ot.Graph("%s, size=%d" % (title, n), r"$X_1$", r"$X_2$")
    # cloud = ot.Cloud(sample)
    cloud = ot.Cloud(sample, "blue", "fsquare", "")
    graph.add(cloud)
    return graph


# %%
# Axial design
# ------------

# %%
levels = [1.0, 1.5, 3.0]
experiment = ot.Axial(2, levels)
sample = experiment.generate()
graph = drawBidimensionalSample(sample, "Axial")
view = otv.View(graph)

# %%
# Use \*= to rescale and += to move a design.
# Pay attention to the grid in the next graph.

# %%
sample *= 2.0
sample += [5.0, 8.0]
graph = drawBidimensionalSample(sample, "Axial")
view = otv.View(graph)

# %%
# Factorial design
# ----------------
#

# %%
experiment = ot.Factorial(2, levels)
sample = experiment.generate()
sample *= 2.0
sample += [5.0, 8.0]
graph = drawBidimensionalSample(sample, "Factorial")
view = otv.View(graph)

# %%
# Composite design
# ----------------
#
# A composite design is a stratified design of experiments enabling to create a pattern as the union of an Axial pattern and a Factorial one.
# The number of points generated is :math:`1 + n_{\mathrm{level}}(2n+2^n)`.
#

# %%
experiment = ot.Composite(2, levels)
sample = experiment.generate()
sample *= 2.0
sample += [5.0, 8.0]
graph = drawBidimensionalSample(sample, "Composite")
view = otv.View(graph)

# %%
# Grid design
# -----------
#

# %%
levels = [3, 4]
experiment = ot.Box(levels)
sample = experiment.generate()
sample *= 2.0
sample += [5.0, 8.0]
graph = drawBidimensionalSample(sample, "Box")
view = otv.View(graph)

# %%
# Display all graphs
otv.View.ShowAll()



"""
Use case : two degree-of-fredom primary/secondary damped oscillator
===================================================================
"""

import openturns as ot


class Oscillator:
    """
    Data class for the oscillator example.

    Attributes
    ----------

    dim : int
        dim = 8, dimension of the problem

    model : :class:`~openturns.SymbolicFunction`
        The limit state function

    muMp : float
        muMp = 1.5, mean of the mass of the primary system

    sigmaOverMuMp : float
        sigmaOverMuMp = 0.1, coefficient of variation of the mass of the primary system

    distributionMp : :class:`~openturns.LogNormal`
        Distribution of the mass of the primary system
        distributionMp = ot.LogNormalMuSigmaOverMu(muMp, sigmaOverMuMp).getDistribution()

    muMs : float
        muMs = 0.01, mean of the mass of the primary system

    sigmaOverMuMs : float
        sigmaOverMuMs = 0.1, coefficient of variation of the mass of the primary system

    distributionMs : :class:`~openturns.LogNormal`
        Distribution of the mass of the secondary system
        distributionMs = ot.LogNormalMuSigmaOverMu(muMs, sigmaOverMuMs).getDistribution()

    muKp : float
        muKp = 1, mean of the spring stiffness of the primary system

    sigmaOverMuKp : float
        sigmaOverMuKp = 0.2, coefficient of variation of the spring stiffness of the primary system

    distributionKp : :class:`~openturns.LogNormal`
        Distribution of the spring stiffness of the primary system
        distributionKp = ot.LogNormalMuSigmaOverMu(muKp, sigmaOverMuKp).getDistribution()

    muKs : float
        muKs = 0.01, mean of the spring stiffness of the secondary system

    sigmaOverMuKs : float
        sigmaOverMuKs = 0.2, coefficient of variation of the spring stiffness of the secondary system

    distributionKs : :class:`~openturns.LogNormal`
        Distribution of the spring stiffness of the secondary system
        distributionKs = ot.LogNormalMuSigmaOverMu(muKs, sigmaOverMuKs).getDistribution()

    muZetap : float
        muZetap = 0.05, mean of the damping ratio of the primary system

    sigmaOverMuZetap : float
        sigmaOverMuZetap = 0.4, coefficient of variation of the damping ratio of the primary system

    distributionZetap : :class:`~openturns.LogNormal`
        Distribution of the damping ratio of the primary system
        distributionZetap = ot.LogNormalMuSigmaOverMu(muZetap, sigmaOverMuZetap).getDistribution()

    muZetas : float
        muZetas = 0.02, mean of the damping ratio of the secondary system

    sigmaOverMuZetas : float
        sigmaOverMuZetas = 0.5, coefficient of variation of the damping ratio of the secondary system

    distributionZetas : :class:`~openturns.LogNormal`
        Distribution of the damping ratio of the secondary system
        distributionZetas = ot.LogNormalMuSigmaOverMu(muZetas, sigmaOverMuZetas).getDistribution()

    muFs : float
        muFs = 27.5, mean of the loading capacity of the secondary spring

    sigmaOverFs : float
        sigmaOverFs = 0.1, coefficient of variation of the loading capacity of the secondary spring

    distributionFs : :class:`~openturns.LogNormal`
        Distribution of the loading capacity of the secondary spring
        distributionFs = ot.LogNormalMuSigmaOverMu(muFs, sigmaOverFs).getDistribution()

    muS0 : float
        muS0 = 100, mean of the intensity of the white noise

    sigmaOverS0 : float
        sigmaOverS0 = 0.1, coefficient of variation of the intensity of the white noise

    distributionS0 : :class:`~openturns.LogNormal`
        Distribution of the intensity of the white noise
        distributionS0 = ot.LogNormalMuSigmaOverMu(muS0, sigmaOverS0).getDistribution()

    distribution : :class:`~openturns.JointDistribution`
        The joint distribution of the input parameters

    Examples
    --------
    >>> from openturns.usecases import oscillator
    >>> # Load the oscillator
    >>> osc = oscillator.Oscillator()
    """

    def __init__(self):
        self.dim = 8

        # Random variable : mp
        self.muMp = 1.5
        self.sigmaOverMuMp = 0.1

        # Random variable : ms
        self.muMs = 0.01
        self.sigmaOverMuMs = 0.1

        # Random variable : kp
        self.muKp = 1.0
        self.sigmaOverMuKp = 0.2

        # Random variable : ks
        self.muKs = 0.01
        self.sigmaOverMuKs = 0.2

        # Random variable : zetap
        self.muZetap = 0.05
        self.sigmaOverMuZetap = 0.4

        # Random variable : zetas
        self.muZetas = 0.02
        self.sigmaOverMuZetas = 0.5

        # Random variable : Fs
        self.muFs = 27.5
        self.sigmaOverMuFs = 0.1

        # Random variable : S0
        self.muS0 = 100.0
        self.sigmaOverMuS0 = 0.1

        # create the limit state function model
        formula = "Fs - 3 * ks * sqrt("
        formula += "pi_ * S0 * (zetap/2 + zetas/2) * zetas * (zetap * sqrt(kp / mp)^3 +"
        formula += "zetas * sqrt(ks / ms)^3) * sqrt(kp / mp) /"
        formula += "(4 * zetas * sqrt(ks / ms)^3 * (zetap * zetas * (4 * (zetap/2 + zetas/2)^2 +"
        formula += "((sqrt(kp / mp) - sqrt(ks / ms)) / (sqrt(kp / mp)/2 + sqrt(ks / ms)/2))^2) +"
        formula += "(ms/mp) * (zetap/2 + zetas/2)^2) * 4 * (zetap/2 + zetas/2) *"
        formula += "(sqrt(kp / mp)/2 + sqrt(ks / ms)/2)^4))"

        self.model = ot.SymbolicFunction(
            ["mp", "ms", "kp", "ks", "zetap", "zetas", "Fs", "S0"], [formula]
        )

        # Mass of primary system
        self.distributionMp = ot.LogNormalMuSigmaOverMu(
            self.muMp, self.sigmaOverMuMp
        ).getDistribution()
        self.distributionMp.setName("Mass of primary system")
        self.distributionMp.setDescription(["mp"])

        # Mass of secondary system
        self.distributionMs = ot.LogNormalMuSigmaOverMu(
            self.muMs, self.sigmaOverMuMs
        ).getDistribution()
        self.distributionMs.setName("Mass of secondary system")
        self.distributionMs.setDescription(["ms"])

        # Spring stiffness of primary system
        self.distributionKp = ot.LogNormalMuSigmaOverMu(
            self.muKp, self.sigmaOverMuKp
        ).getDistribution()
        self.distributionKp.setName("Spring stiffness of primary system")
        self.distributionKp.setDescription(["kp"])

        # Spring stiffness of secondary system
        self.distributionKs = ot.LogNormalMuSigmaOverMu(
            self.muKs, self.sigmaOverMuKs
        ).getDistribution()
        self.distributionKs.setName("Spring stiffness of secondary system")
        self.distributionKs.setDescription(["ks"])

        # Damping ratio of primary system
        self.distributionZetap = ot.LogNormalMuSigmaOverMu(
            self.muZetap, self.sigmaOverMuZetap
        ).getDistribution()
        self.distributionZetap.setName("Damping ratio of primary system")
        self.distributionZetap.setDescription(["zetap"])

        # Damping ratio of secondary system
        self.distributionZetas = ot.LogNormalMuSigmaOverMu(
            self.muZetas, self.sigmaOverMuZetas
        ).getDistribution()
        self.distributionZetas.setName("Damping ratio of secondary system")
        self.distributionZetas.setDescription(["zetas"])

        # Force capacity of secondary spring
        self.distributionFs = ot.LogNormalMuSigmaOverMu(
            self.muFs, self.sigmaOverMuFs
        ).getDistribution()
        self.distributionFs.setName("Force capacity of secondary spring")
        self.distributionFs.setDescription(["Fs"])

        # Intensity of white noise excitation
        self.distributionS0 = ot.LogNormalMuSigmaOverMu(
            self.muS0, self.sigmaOverMuS0
        ).getDistribution()
        self.distributionS0.setName("Intensity of white noise excitation")
        self.distributionS0.setDescription(["S0"])

        # Joint distribution of the input parameters
        self.distribution = ot.JointDistribution(
            [
                self.distributionMp,
                self.distributionMs,
                self.distributionKp,
                self.distributionKs,
                self.distributionZetap,
                self.distributionZetas,
                self.distributionFs,
                self.distributionS0,
            ]
        )



#! /usr/bin/env python

import openturns as ot
import openturns.testing as ott
from math import sqrt, pi, exp, log

ot.TESTPREAMBLE()

ot.ResourceMap.SetAsUnsignedInteger(
    "LinearCombinationDistribution-DefaultMaxSize", 4000000
)
# Deactivate the simplification mechanism as we want to test the Poisson formula
# based algorithm here
ot.ResourceMap.SetAsBool("LinearCombinationDistribution-SimplifyAtoms", False)

# Create a collection of test-cases and the associated references
numberOfTests = 3
testCases = list()
references = list()
testCases.append([ot.Uniform(-1.0, 3.0)] * 2)
references.append(ot.Triangular(-2.0, 2.0, 6.0))
testCases.append([ot.Normal(), ot.Normal(1.0, 2.0), ot.Normal(-2.0, 2.0)])
references.append(ot.Normal(-1.0, 3.0))
testCases.append([ot.Exponential()] * 3)
references.append(ot.Gamma(3.0, 1.0, 0.0))
print("testCases=", testCases)
print("references=", references)
for testIndex in range(len(testCases)):
    # Instantiate one distribution object
    distribution = ot.LinearCombinationDistribution(testCases[testIndex])
    distribution.setBlockMin(5)
    distribution.setBlockMax(20)
    distributionReference = references[testIndex]
    print("Distribution ", repr(distribution))
    print("Distribution ", distribution)

    # Is this distribution elliptical ?
    print("Elliptical = ", distribution.isElliptical())

    # Is this distribution continuous ?
    print("Continuous = ", distribution.isContinuous())

    # Test for realization of distribution
    oneRealization = distribution.getRealization()
    print("oneRealization=", oneRealization)

    # Test for sampling
    size = 10000
    oneSample = distribution.getSample(size)
    print("oneSample first=", oneSample[0], " last=", oneSample[size - 1])
    print("mean=", oneSample.computeMean())
    print("covariance=", oneSample.computeCovariance())

    # Define a point
    point = ot.Point(distribution.getDimension(), 0.5)
    print("Point= ", point)

    # Show PDF and CDF of point
    DDF = distribution.computeDDF(point)[0]
    print("ddf      =%.5g" % DDF)
    print("ddf (ref)=", distributionReference.computeDDF(point))
    PDF = distribution.computePDF(point)
    print("pdf      =%.6f" % PDF)
    print("pdf (ref)=%.6f" % distributionReference.computePDF(point))
    CDF = distribution.computeCDF(point)
    print("cdf      =%.6f" % CDF)
    print("cdf (ref)=%.6f" % distributionReference.computeCDF(point))
    CF = distribution.computeCharacteristicFunction(point[0])
    print("characteristic function=%.6f + %.6fi" % (CF.real, CF.imag))
    LCF = distribution.computeLogCharacteristicFunction(point[0])
    print("log characteristic function=%.6f + %.6fi" % (LCF.real, LCF.imag))
    quantile = distribution.computeQuantile(0.95)
    print("quantile      =", quantile)
    print("quantile (ref)=", distributionReference.computeQuantile(0.95))
    print("cdf(quantile)=%.6f" % distribution.computeCDF(quantile))
    quantileComp = distribution.computeQuantile(0.95, True)
    print("quantile comp.=", quantileComp)
    print(
        "cdfComp(quantileComp)=%.6f"
        % distribution.computeComplementaryCDF(quantileComp)
    )
    # Get 95% survival function
    inverseSurvival = ot.Point(distribution.computeInverseSurvivalFunction(0.95))
    print("InverseSurvival=", repr(inverseSurvival))
    print(
        "Survival(inverseSurvival)=%.6f"
        % distribution.computeSurvivalFunction(inverseSurvival)
    )
    # Entropy: too expansive for now...
    if False:
        print("entropy=%.6f" % distribution.computeEntropy())

    # Confidence regions: too expansive for now...
    if False:
        print("dimension=", distribution.getDimension(), "test case=", testIndex)
        (
            interval,
            threshold,
        ) = distribution.computeMinimumVolumeIntervalWithMarginalProbability(0.95)
        print("Minimum volume interval=", interval)
        print("threshold=", ot.Point(1, threshold))
        levelSet, beta = distribution.computeMinimumVolumeLevelSetWithThreshold(0.95)
        print("Minimum volume level set=", levelSet)
        print("beta=", ot.Point(1, beta))
        (
            interval,
            beta,
        ) = distribution.computeBilateralConfidenceIntervalWithMarginalProbability(0.95)
        print("Bilateral confidence interval=", interval)
        print("beta=", ot.Point(1, beta))
        (
            interval,
            beta,
        ) = distribution.computeUnilateralConfidenceIntervalWithMarginalProbability(
            0.95, False
        )
        print("Unilateral confidence interval (lower tail)=", interval)
        print("beta=", ot.Point(1, beta))
        (
            interval,
            beta,
        ) = distribution.computeUnilateralConfidenceIntervalWithMarginalProbability(
            0.95, True
        )
        print("Unilateral confidence interval (upper tail)=", interval)
        print("beta=", ot.Point(1, beta))

    mean = distribution.getMean()
    print("mean      =", mean)
    print("mean (ref)=", distributionReference.getMean())
    standardDeviation = distribution.getStandardDeviation()
    print("standard deviation      =", standardDeviation)
    print("standard deviation (ref)=", distributionReference.getStandardDeviation())
    skewness = distribution.getSkewness()
    print("skewness      =", skewness)
    print("skewness (ref)=", distributionReference.getSkewness())
    kurtosis = distribution.getKurtosis()
    print("kurtosis      =", kurtosis)
    print("kurtosis (ref)=", distributionReference.getKurtosis())
    covariance = distribution.getCovariance()
    print("covariance      =", covariance)
    print("covariance (ref)=", distributionReference.getCovariance())
    parameters = distribution.getParametersCollection()
    print("parameters=", parameters)
    print("Standard representative=", distribution.getStandardRepresentative())
    print("blockMin=", distribution.getBlockMin())
    print("blockMax=", distribution.getBlockMax())
    print("maxSize=", distribution.getMaxSize())
    print("alpha=", distribution.getAlpha())
    print("beta=", distribution.getBeta())

    ot.Log.Show(ot.Log.TRACE)
    validation = ott.DistributionValidation(distribution)
    validation.skipEntropy()  # slow
    validation.skipMinimumVolumeLevelSet()  # slow
    validation.skipTransformation()  # transformation accuracy is a bit low
    validation.run()

# Tests of the simplification mechanism
weights = ot.Point(0)
coll = ot.DistributionCollection(0)
coll.add(ot.Dirac(0.5))
weights.add(1.0)
coll.add(ot.Normal(1.0, 2.0))
weights.add(2.0)
coll.add(ot.Normal(2.0, 1.0))
weights.add(-3.0)
coll.add(ot.Uniform(-2.0, 2.0))
weights.add(-1.0)
coll.add(ot.Uniform(2.0, 4.0))
weights.add(2.0)
coll.add(ot.Exponential(2.0, -3.0))
weights.add(1.5)
rm = ot.LinearCombinationDistribution(coll, weights)
coll.add(rm)
weights.add(-2.5)
coll.add(ot.Gamma(3.0, 4.0, -2.0))
weights.add(2.5)
distribution = ot.LinearCombinationDistribution(coll, weights)
print("distribution=", repr(distribution))
print("distribution=", distribution)
mu = distribution.getMean()[0]
sigma = distribution.getStandardDeviation()[0]
for i in range(10):
    x = mu + (-3.0 + 6.0 * i / 9.0) * sigma
    print("pdf( %.6f )=%.6f" % (x, distribution.computePDF(x)))

# Tests of the projection mechanism
collFactories = [
    ot.UniformFactory(),
    ot.NormalFactory(),
    ot.TriangularFactory(),
    ot.ExponentialFactory(),
    ot.GammaFactory(),
]
# , TrapezoidalFactory()
result, norms = distribution.project(collFactories)
print("projections=", result)
print("norms=", norms)
# ------------------------------ Multivariate tests ------------------------------#
# 2D LinearCombinationDistribution
collection = [ot.Normal(0.0, 1.0)] * 3

weightMatrix = ot.Matrix(2, 3)
weightMatrix[0, 0] = 1.0
weightMatrix[0, 1] = -2.0
weightMatrix[0, 2] = 1.0
weightMatrix[1, 0] = 1.0
weightMatrix[1, 1] = 1.0
weightMatrix[1, 2] = -3.0

# Build the LinearCombinationDistribution
distribution2D = ot.LinearCombinationDistribution(collection, weightMatrix)
print("distribution = ", distribution2D)
print("range = ", distribution2D.getRange())
print("mean = ", distribution2D.getMean())
print("cov = ", distribution2D.getCovariance())
print("sigma = ", distribution2D.getStandardDeviation())
distribution2D.setBlockMin(3)
distribution2D.setBlockMax(10)

# Build a grid for validation
xMin = distribution2D.getRange().getLowerBound()[0]
xMax = distribution2D.getRange().getUpperBound()[0]
yMin = distribution2D.getRange().getLowerBound()[1]
yMax = distribution2D.getRange().getUpperBound()[1]
# Number of points of discretization
nx = 4
ny = 4
boxParameters = [nx, ny]
boxGrid = ot.Box(boxParameters)
grid = boxGrid.generate()
# scaling box grid
scaleFactor = [0.25 * (xMax - xMin), 0.25 * (yMax - yMin)]
grid *= scaleFactor
# translating
translateFactor = distribution2D.getMean()[0:2]
grid += translateFactor
# Compute PDF
# parameters for theoritical PDF, obtained thanks to Maple
factor = sqrt(2) / (20 * pi)
for index in range(grid.getSize()):
    point = grid[index]
    PDF = distribution2D.computePDF(point)
    # Very small values are not very accurate on x86, skip them
    if PDF < 1.0e-12:
        continue
    print("pdf      = %.6g" % PDF)
    x, y = tuple(point)
    pdf_ref = factor * exp(-3.0 / 50.0 * y * y - 2.0 / 25 * x * y - 11.0 / 100 * x * x)
    print("pdf (ref)= %.6g" % pdf_ref)

# 2D test, but too much CPU consuming
collUniforme = [ot.Uniform(0, 1)] * 3
# Build the LinearCombinationDistribution
dist_2D = ot.LinearCombinationDistribution(collUniforme, weightMatrix)
dist_2D.setBlockMin(3)
dist_2D.setBlockMax(8)

print("new distribution = ", dist_2D)
print("range = ", dist_2D.getRange())
print("mean = ", dist_2D.getMean())
print("cov = ", dist_2D.getCovariance())
print("sigma = ", dist_2D.getStandardDeviation())

# Discretization on grid mu, mu + \sigma
newGrid = boxGrid.generate()
# scaling box grid
newGrid *= dist_2D.getStandardDeviation()
# translating
newGrid += dist_2D.getMean()
# Compute PDF
for index in range(newGrid.getSize()):
    point = newGrid[index]
    PDF = dist_2D.computePDF(point)
    print("pdf      = %.6g" % PDF)

# 3D test
ot.ResourceMap.SetAsUnsignedInteger(
    "LinearCombinationDistribution-DefaultMaxSize", 8290688
)
mixture = ot.Mixture([ot.Normal(2, 1), ot.Normal(-2, 1)])
collection = [ot.Normal(0.0, 1.0), mixture, ot.Uniform(0, 1), ot.Uniform(0, 1)]
matrix = ot.Matrix([[1, -0.05, 1, -0.5], [0.5, 1, -0.05, 0.3], [-0.5, -0.1, 1.2, -0.8]])
dist_3D = ot.LinearCombinationDistribution(collection, matrix)
dist_3D.setBlockMin(3)
dist_3D.setBlockMax(6)

print("3D distribution = ", dist_3D)
print("range = ", dist_3D.getRange())
print("mean = ", dist_3D.getMean())
print("cov = ", dist_3D.getCovariance())
print("sigma = ", dist_3D.getStandardDeviation())
# Total number of points (is (2+2)**3)
# Test is CPU consuming
N = 2
b = ot.Box([N, N, N])
# Grid ==> (mu, mu+sigma)
grid3D = b.generate() * dist_3D.getStandardDeviation() + dist_3D.getMean()

for i in range(grid3D.getSize() // 4):
    point = grid3D[4 * i]
    PDF = dist_3D.computePDF(point)
    print("pdf      = %.6g" % PDF)

# For ticket 882
mixture = ot.LinearCombinationDistribution([ot.Dirac()])
graph = mixture.drawPDF()
graph = mixture.drawCDF()

# Test computeQuantile for the specific case of an analytical 1D mixture
case1 = 0.1 * ot.ChiSquare()
q = case1.computeQuantile(0.95)[0]
print("case 1, q=%.6f" % q)
q = case1.computeQuantile(0.95, True)[0]
print("case 1, q comp=%.6f" % q)
case2 = -0.1 * ot.ChiSquare()
q = case2.computeQuantile(0.95)[0]
print("case 2, q=%.6f" % q)
q = case2.computeQuantile(0.95, True)[0]
print("case 2, q comp=%.6f" % q)
# For ticket 953
atom1 = ot.TruncatedDistribution(ot.Uniform(0.0, 1.0), 0.0, 1.0)
atom2 = ot.Uniform(0.0, 2.0)
sum = atom1 + atom2
print("sum=", sum)
print("CDF=%.6g" % sum.computeCDF(2.0))
print("quantile=", sum.computeQuantile(0.2))
minS = 0.2
maxS = 10.0
muS = (log(minS) + log(maxS)) / 2.0
sigma = (log(maxS) - muS) / 3.0
atom1 = ot.TruncatedDistribution(ot.LogNormal(muS, sigma), minS, maxS)
atom2 = ot.Uniform(0.0, 2.0)
sum = atom1 + atom2
print("sum=", sum)
print("CDF=%.6g" % sum.computeCDF(2.0))
print("quantile=", sum.computeQuantile(0.2))
# For ticket 1129
dist = ot.LinearCombinationDistribution([ot.Uniform()] * 200)
print("CDF(0)=%.5g" % dist.computeCDF([0]))

# check parameter accessors
dist = ot.Gumbel() + ot.Normal(0, 0.1)
print("before", dist)
p = [1849.41, -133.6, -133.6, 359.172]
dist.setParameter(p)
assert p == dist.getParameter(), "wrong parameters"
print("after ", dist)

# alias
dist = ot.RandomMixture([ot.Uniform()] * 3)



"""
Draw minimum volume level sets
==============================
"""

# %%
import openturns as ot
import openturns.viewer as otv


# %%
# Draw minimum volume level set in 1D
# -----------------------------------
#
# In this paragraph, we compute the minimum volume level set of a univariate distribution.


# %%
# With a Normal, minimum volume LevelSet
# ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

# %%
n = ot.Normal()

# %%
graph = n.drawPDF()
view = otv.View(graph)

# %%
# We want to compute the minimum volume `LevelSet` which contains `alpha` = 90% of the distribution.
# The `threshold` is the value of the PDF corresponding the `alpha`-probability:
# the points contained in the `LevelSet` have a PDF value lower or equal to this threshold.

# %%
alpha = 0.9
levelSet, threshold = n.computeMinimumVolumeLevelSetWithThreshold(alpha)
threshold

# %%
# The `LevelSet` has a `contains` method. Obviously, the point 0 is in the LevelSet.

# %%
levelSet.contains([0.0])


# %%
def computeSampleInLevelSet(distribution, levelSet, sampleSize=1000):
    """
    Generate a sample from given distribution.
    Extract the sub-sample which is contained in the levelSet.
    """
    sample = distribution.getSample(sampleSize)
    dim = distribution.getDimension()
    # Get the list of points in the LevelSet.
    inLevelSet = []
    for x in sample:
        if levelSet.contains(x):
            inLevelSet.append(x)
    # Extract the sub-sample of the points in the LevelSet
    numberOfPointsInLevelSet = len(inLevelSet)
    inLevelSetSample = ot.Sample(numberOfPointsInLevelSet, dim)
    for i in range(numberOfPointsInLevelSet):
        inLevelSetSample[i] = inLevelSet[i]
    return inLevelSetSample


# %%
def from1Dto2Dsample(oldSample):
    """
    Create a 2D sample from a 1D sample with zero ordinate (for the graph).
    """
    size = oldSample.getSize()
    newSample = ot.Sample(size, 2)
    for i in range(size):
        newSample[i, 0] = oldSample[i, 0]
    return newSample


# %%
def drawLevelSet1D(distribution, levelSet, alpha, threshold, sampleSize=100):
    """
    Draw a 1D sample included in a given levelSet.
    The sample is generated from the distribution.
    """
    inLevelSample = computeSampleInLevelSet(distribution, levelSet, sampleSize)
    cloudSample = from1Dto2Dsample(inLevelSample)
    graph = distribution.drawPDF()
    mycloud = ot.Cloud(cloudSample)
    graph.add(mycloud)
    graph.setTitle(
        "%.2f%% of the distribution, sample size = %d, " % (100 * alpha, sampleSize)
    )
    return graph


# %%
graph = drawLevelSet1D(n, levelSet, alpha, threshold)
view = otv.View(graph)

# %%
# With a Normal, minimum volume Interval
# ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

# %%
interval = n.computeMinimumVolumeInterval(alpha)
interval


# %%
def drawPDFAndInterval1D(distribution, interval, alpha):
    """
    Draw the PDF of the distribution and the lower and upper bounds of an interval.
    """
    xmin = interval.getLowerBound()[0]
    xmax = interval.getUpperBound()[0]
    graph = distribution.drawPDF()
    yvalue = distribution.computePDF(xmin)
    curve = ot.Curve([[xmin, 0.0], [xmin, yvalue], [xmax, yvalue], [xmax, 0.0]])
    curve.setColor("black")
    graph.add(curve)
    graph.setTitle(
        "%.2f%% of the distribution, lower bound = %.3f, upper bound = %.3f"
        % (100 * alpha, xmin, xmax)
    )
    return graph


# %%
# The `computeMinimumVolumeInterval` returns an `Interval`.

# %%
graph = drawPDFAndInterval1D(n, interval, alpha)
view = otv.View(graph)

# %%
# With a Mixture, minimum volume LevelSet
# ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

# %%
m = ot.Mixture([ot.Normal(-5.0, 1.0), ot.Normal(5.0, 1.0)], [0.2, 0.8])

# %%
graph = m.drawPDF()
view = otv.View(graph)

# %%
alpha = 0.9
levelSet, threshold = m.computeMinimumVolumeLevelSetWithThreshold(alpha)
threshold

# %%
# The interesting point is that a `LevelSet` may be non-contiguous. In the current mixture example, this is not an interval.

# %%
graph = drawLevelSet1D(m, levelSet, alpha, threshold, 1000)
view = otv.View(graph)

# %%
# With a Mixture, minimum volume Interval
# ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

# %%
interval = m.computeMinimumVolumeInterval(alpha)
interval

# %%
# The `computeMinimumVolumeInterval` returns an `Interval`. The bounds of this interval are different from the previous `LevelSet`.

# %%
graph = drawPDFAndInterval1D(m, interval, alpha)
view = otv.View(graph)


# %%
# Draw minimum volume level set in 2D
# -----------------------------------
#
# In this paragraph, we compute the minimum volume level set of a bivariate distribution.


# %%
# Create a Normal distribution
corr = ot.CorrelationMatrix(2)
corr[0, 1] = 0.2
copula = ot.NormalCopula(corr)
x1 = ot.Normal(-1.0, 1)
x2 = ot.Normal(2, 1)
x_funk = ot.JointDistribution([x1, x2], copula)

# Create a second Normal distribution
x1 = ot.Normal(1.0, 1)
x2 = ot.Normal(-2, 1)
x_punk = ot.JointDistribution([x1, x2], copula)

# Mix the distributions
mixture = ot.Mixture([x_funk, x_punk], [0.5, 1.0])

# %%
ot.ResourceMap.SetAsString("Contour-DefaultColorMapNorm", "log")
graph = mixture.drawPDF()
view = otv.View(graph)

# %%
# For a multivariate distribution (with dimension greater than 1), the `computeMinimumVolumeLevelSetWithThreshold` uses Monte-Carlo sampling.

# %%
ot.ResourceMap.SetAsUnsignedInteger(
    "Distribution-MinimumVolumeLevelSetSamplingSize", 1000
)

# %%
# We want to compute the minimum volume `LevelSet` which contains `alpha` = 90% of the distribution.
# The `threshold` is the value of the PDF corresponding the `alpha`-probability:
# the points contained in the LevelSet have a PDF value lower or equal to this threshold.

# %%
alpha = 0.9
levelSet, threshold = mixture.computeMinimumVolumeLevelSetWithThreshold(alpha)
threshold


# %%
def drawLevelSetContour2D(distribution, alpha, level_set, threshold, sampleSize=500):
    """
    Compute the minimum volume LevelSet of measure equal to alpha and get the
    corresponding density value (named threshold).
    Generate a sample of the distribution and draw it.
    Draw a contour plot for the distribution, where the PDF is equal to threshold.
    """
    sample = distribution.getSample(sampleSize)
    graph = ot.VisualTest.DrawInsideOutside(level_set, sample).getGraph(0, 0)
    labels = ["%.2f%%" % (100 * alpha)]
    contour_graph = distribution.drawPDF(sample.getMin(), sample.getMax())
    contour = contour_graph.getDrawable(0).getImplementation()
    contour.setLevels([threshold])
    contour.setLabels(labels)
    contour.setDrawLabels(True)
    contour.setColor("black")
    graph.setTitle(
        "%.2f%% of the distribution, sample size = %d" % (100 * alpha, sampleSize)
    )
    graph.add(contour)
    return graph, contour


# %%
# The following plot shows that 90% of the sample is contained in the `LevelSet`.

# %%
graph, contour = drawLevelSetContour2D(mixture, alpha, levelSet, threshold)
view = otv.View(graph)

# %%
otv.View.ShowAll()



#! /usr/bin/env python

import openturns as ot
import os

ot.TESTPREAMBLE()

f = ot.Function()

# load
study = ot.Study()
study.setStorageManager(ot.XMLStorageManager("pyf.xml"))
study.load()
study.fillObject("f", f)

x = [4, 5]
print(f(x))

os.remove("pyf.xml")



#! /usr/bin/env python

import openturns as ot
import openturns.testing as ott

ot.TESTPREAMBLE()


# Problem parameters
inputDimension = 2
outputDimension = 1
rho = 0.3
a = 4.0
b = 5.0

# Reference analytical values
covTh = a * a + b * b + 2 * a * b * rho
Si = [
    [(a * a + a * b * rho) / covTh, a * a / covTh],
    [(b * b + a * b * rho) / covTh, b * b / covTh],
]

# Model
inputName = ["X1", "X2", "a", "b"]
formula = ["a * X1 + b * X2"]

full = ot.SymbolicFunction(inputName, formula)
model = ot.ParametricFunction(full, [2, 3], [a, b])

# Input distribution
distribution = ot.JointDistribution([ot.Normal()] * inputDimension)

# Correlated input distribution
S = ot.CorrelationMatrix(inputDimension)
S[1, 0] = 0.3
R = ot.NormalCopula().GetCorrelationFromSpearmanCorrelation(S)
myCopula = ot.NormalCopula(R)
myCorrelatedInputDistribution = ot.JointDistribution(
    [ot.Normal()] * inputDimension, myCopula
)

sample = myCorrelatedInputDistribution.getSample(2000)

# Orthogonal basis
enumerateFunction = ot.LinearEnumerateFunction(inputDimension)
productBasis = ot.OrthogonalProductPolynomialFactory(
    [ot.HermiteFactory()] * inputDimension, enumerateFunction
)
# Adaptive strategy
adaptiveStrategy = ot.FixedStrategy(
    productBasis, enumerateFunction.getStrataCumulatedCardinal(4)
)
# x/y samples
samplingSize = 250
input_sample = distribution.getSample(samplingSize)
output_sample = model(input_sample)

# Polynomial chaos algorithm
algo = ot.FunctionalChaosAlgorithm(
    input_sample, output_sample, distribution, adaptiveStrategy
)
algo.run()

# Post-process the results
result = ot.FunctionalChaosResult(algo.getResult())
ancova = ot.ANCOVA(result, sample)
indices = ancova.getIndices()
uncorrelatedIndices = ancova.getUncorrelatedIndices()

for i in range(inputDimension):
    value = indices[i]
    print(
        "ANCOVA index",
        i,
        "= %.6g" % value,
        "absolute error=%.6g" % abs(value - Si[i][0]),
    )
    value = uncorrelatedIndices[i]
    print(
        "ANCOVA uncorrelated index",
        i,
        "= %.8f" % value,
        "absolute error=%.6g" % abs(value - Si[i][1]),
    )

# Compare full/sparse
ot.RandomGenerator.SetSeed(323)

model = ot.SymbolicFunction(["x1", "x2", "x3"], ["x1 + 0.56*x2 + 0.9*x3"])
distribution = ot.Normal(3)
distribution.setDescription(["x1", "x2", "x3"])

n_sample = 10
input_sample = distribution.getSample(n_sample)
output_sample = model(input_sample)

dim = 3
enumerateFunction = ot.LinearEnumerateFunction(dim)
polyCol = [0.0] * dim
for i in range(dim):
    polyCol[i] = ot.StandardDistributionPolynomialFactory(distribution.getMarginal(i))

# Chaos definition
multivariateBasis = ot.OrthogonalProductPolynomialFactory(polyCol, enumerateFunction)
indexMax = enumerateFunction.getStrataCumulatedCardinal(1)
strategy = ot.FixedStrategy(multivariateBasis, indexMax)

approximation_algorithm = ot.LeastSquaresMetaModelSelectionFactory(
    ot.LARS(), ot.CorrectedLeaveOneOut()
)
evaluationStrategy_sparse = ot.LeastSquaresStrategy(approximation_algorithm)
evaluationStrategy = ot.LeastSquaresStrategy()

# sparse and not sparse
chaos = ot.FunctionalChaosAlgorithm(
    input_sample, output_sample, distribution, strategy, evaluationStrategy
)
chaos.run()
chaos_sparse = ot.FunctionalChaosAlgorithm(
    input_sample, output_sample, distribution, strategy, evaluationStrategy_sparse
)
chaos_sparse.run()

ancova = ot.ANCOVA(chaos.getResult(), input_sample)
ancova_sparse = ot.ANCOVA(chaos_sparse.getResult(), input_sample)
print(
    "Indice ancova, chaos normal : {:0.3f} {:0.3f} {:0.3f}".format(*ancova.getIndices())
)
print(
    "Indice ancova, chaos sparse : {:0.3f} {:0.3f} {:0.3f}".format(
        *ancova_sparse.getIndices()
    )
)

ott.assert_almost_equal(ancova.getIndices(), ancova_sparse.getIndices())



import openturns as ot
import openturns.viewer as otv

X = ot.RandomVector(ot.Normal())
sample = X.getSample(200)
histogram = ot.HistogramFactory().build(sample).drawPDF()
histogram.setTitle("X~N(0,1)")
otv.View(histogram, figure_kw={"figsize": (6, 4)}, add_legend=False)



import openturns as ot
from matplotlib import pyplot as plt
import openturns.viewer as otv

distXgivenT = ot.Exponential()
distGamma = ot.Uniform(1.0, 2.0)
distLambda = ot.Uniform(0.0, 1)
distTheta = ot.JointDistribution([distLambda, distGamma])
rvTheta = ot.RandomVector(distTheta)

rvX = ot.CompoundRandomVector(distXgivenT, rvTheta)
sampleX = rvX.getSample(1000)

histX = ot.HistogramFactory().build(sampleX)
graph = histX.drawPDF()
graph.setXTitle("x")
graph.setYTitle("pdf")
graph.setTitle(
    r"Conditional Random Vector: Exp( $\lambda$, $\gamma$), $\lambda \sim \mathcal{U}(0,1)$, $\gamma \sim \mathcal{U}(1,2)$"
)

fig = plt.figure(figsize=(8, 4))
axis = fig.add_subplot(111)
axis.set_xlim(auto=True)

otv.View(graph, figure=fig, axes=[axis], add_legend=False)



#! /usr/bin/env python

import openturns as ot
import openturns.experimental as ote
from openturns.testing import assert_almost_equal


ot.TESTPREAMBLE()


g = ot.SymbolicFunction(["x"], ["sin(x)"])
x_train = ot.Sample([[x] for x in [1.0, 3.0, 4.0, 6.0, 7.9, 11.0, 11.5]])
y_train = g(x_train)

basis = ot.ConstantBasisFactory(1).build()
covarianceModel = ot.MaternModel([1.0], 1.5)

fitter_algo = ot.GaussianProcessFitter(x_train, y_train, covarianceModel, basis)
fitter_algo.run()
fitter_result = fitter_algo.getResult()

gpr_algo = ot.GaussianProcessRegression(fitter_result)
gpr_algo.run()
gpr_result = gpr_algo.getResult()


# Naive cross-validation
cv_naive_residuals = ot.Sample(x_train.getSize(), 1)
fitted_covariance_model = gpr_result.getCovarianceModel()
fitted_covariance_model.setActiveParameter([])  # no parameter optimization in CV
for i in range(x_train.getSize()):
    # Create the training sample without the i-th point
    x_train_cv = ot.Sample(x_train)
    x_train_cv.erase(i)
    y_train_cv = ot.Sample(y_train)
    y_train_cv.erase(i)
    # Build the GPR model
    fitter_algo_cv = ot.GaussianProcessFitter(
        x_train_cv, y_train_cv, fitted_covariance_model, basis
    )
    fitter_algo_cv.run()
    fitter_result_cv = fitter_algo_cv.getResult()
    gpr_algo_cv = ot.GaussianProcessRegression(fitter_result_cv)
    gpr_algo_cv.run()
    gpr_result_cv = gpr_algo_cv.getResult()
    gpr_predictor_cv = gpr_result_cv.getMetaModel()
    # Evaluate the residual at the i-th point
    cv_naive_residuals[i, 0] = y_train[i, 0] - gpr_predictor_cv(x_train[i])[0]

# OT cross-validation
cv = ote.GaussianProcessRegressionCrossValidation(
    gpr_result, ot.LeaveOneOutSplitter(x_train.getSize())
)
cv_ot_residuals = cv.getResidualSample()

# Check that both implementations give the same result
assert_almost_equal(cv_ot_residuals, cv_naive_residuals, 1.0e-12, 0.0)



#! /usr/bin/env python

import openturns as ot
import openturns.testing as ott
from openturns.usecases import ishigami_function

ot.TESTPREAMBLE()
ot.RandomGenerator.SetSeed(0)

# Ishigami use-case
ishigami = ishigami_function.IshigamiModel()
distX = ishigami.distribution

# Get a sample of it
size = 100
X = distX.getSample(size)


# The Ishigami model
modelIshigami = ishigami.model
modelIshigami.setParameter([5, 0.1])

# Apply model: Y = m(X)
Y = modelIshigami(X)

# We define the covariance models for the HSIC indices.
# For the input, we consider a SquaredExponential covariance model.
covarianceModelCollection = ot.CovarianceModelCollection()

# Input sample
for i in range(3):
    Xi = X.getMarginal(i)
    Cov = ot.SquaredExponential(1)
    Cov.setScale(Xi.computeStandardDeviation())
    covarianceModelCollection.add(Cov)

# Output sample with squared exponential covariance
Cov2 = ot.SquaredExponential(1)
Cov2.setScale(Y.computeStandardDeviation())
covarianceModelCollection.add(Cov2)

# We choose an estimator type :
#  - unbiased: HSICUStat;
#   - biased: HSICVStat.
#
estimatorType = ot.HSICUStat()


# We define a distance function for the weights
#  For the TSA, the critical domain is [5,+inf].
interval = ot.Interval(5, float("inf"))
g = ot.DistanceToDomainFunction(interval)

stdDev = Y.computeStandardDeviation()[0]
foo = ot.SymbolicFunction(["x", "s"], ["exp(-x/s)"])
g2 = ot.ParametricFunction(foo, [1], [0.1 * stdDev])

# The filter function
filterFunction = ot.ComposedFunction(g2, g)

# random generator state
# use the same state for parallel/sequential validation
state = ot.RandomGenerator.GetState()

for key in [True, False]:
    ot.ResourceMap.SetAsBool("HSICEstimator-ParallelPValues", key)
    ot.RandomGenerator.SetState(state)

    # We eventually build the HSIC object!
    TSA = ot.HSICEstimatorTargetSensitivity(
        covarianceModelCollection, X, Y, estimatorType, filterFunction
    )

    # We get the R2-HSIC
    R2HSIC = TSA.getR2HSICIndices()
    ott.assert_almost_equal(R2HSIC, [0.26863688, 0.00468423, 0.00339962])

    # and the HSIC indices
    HSICIndices = TSA.getHSICIndices()
    ott.assert_almost_equal(HSICIndices, [0.00107494, 0.00001868, 0.00001411])

    # We get the asymptotic pvalue
    pvaluesAs = TSA.getPValuesAsymptotic()
    ott.assert_almost_equal(pvaluesAs, [0.00000000, 0.26201467, 0.28227083])

    # We set the number of permutations for the pvalue estimate
    b = 100
    TSA.setPermutationSize(b)

    # We get the pvalue estimate by permutations
    pvaluesPerm = TSA.getPValuesPermutation()
    ott.assert_almost_equal(pvaluesPerm, [0, 0.257426, 0.217822])

    # Change the filter function and recompute everything
    squaredExponential = ot.SymbolicFunction("x", "exp(-0.1 * x^2)")
    alternateFilter = ot.ComposedFunction(squaredExponential, g)
    TSA.setFilterFunction(alternateFilter)
    ott.assert_almost_equal(TSA.getR2HSICIndices(), [0.373511, 0.0130156, 0.0153977])
    ott.assert_almost_equal(
        TSA.getHSICIndices(), [0.00118685, 4.12193e-05, 5.07577e-05], 1e-4, 0.0
    )
    print(TSA.getPValuesPermutation())
    ott.assert_almost_equal(TSA.getPValuesPermutation(), [0.0, 0.118812, 0.158416])
    ott.assert_almost_equal(
        TSA.getPValuesAsymptotic(), [7.32022e-13, 0.143851, 0.128866]
    )



"""
Create and draw multivariate distributions
==========================================
"""

# %%
# In this example we create and draw multidimensional distributions.
import openturns as ot
import openturns.viewer as otv

# %%
# Create a multivariate model with a :class:`~openturns.JointDistribution`
# ------------------------------------------------------------------------
#
# In this paragraph we use :class:`~openturns.JointDistribution` class to
# build a multivariate distribution of dimension :math:`\inputDim`, from:
#
# - :math:`\inputDim` scalar distributions whose cumulative distribution functions are
#   denoted by :math:`(F_1, \dots, F_\inputDim)`, called the  *instrumental marginals*,
# - and a core :math:`K` which is a multivariate distribution of dimension :math:`\inputDim` whose range is
#   included in :math:`[0,1]^\inputDim`.
#
# First case: the core is a copula
# ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
# In this case, we use a core which is a copula. Thus, the instrumental marginals
# are the marginals of the multivariate distribution.
#
# We first create the marginals of the distribution:
#
#   - a :class:`~openturns.Normal` distribution;
#   - a :class:`~openturns.Gumbel` distribution.
#
# We use a :class:`~openturns.ClaytonCopula` as dependence structure.
marginals = [ot.Normal(), ot.Gumbel()]
theta = 2.0
cop = ot.ClaytonCopula(theta)
distribution = ot.JointDistribution(marginals, cop)

# %%
# We can check here that the instrumental marginals really are  the marginal distributions.
# In the following graphs, we draw the instrumental marginals and the real marginals, obtained with
# the method :meth:`~openturns.Distribution.getMarginal`.
# First, we draw the probability density functions of each component.
graph_PDF_0 = marginals[0].drawPDF()
graph_PDF_0.add(distribution.getMarginal(0).drawPDF())
graph_PDF_0.setLegends(["instrumental marg", "marg"])
graph_PDF_0.setTitle("First component")
view = otv.View(graph_PDF_0)

graph_PDF_1 = marginals[1].drawPDF()
graph_PDF_1.add(distribution.getMarginal(1).drawPDF())
graph_PDF_1.setLegends(["instrumental marg", "marg"])
graph_PDF_1.setTitle("Second component")
view = otv.View(graph_PDF_1)

# %%
# Then, we draw the cumulative distribution functions.
graph_CDF_0 = marginals[0].drawCDF()
graph_CDF_0.add(distribution.getMarginal(0).drawCDF())
graph_CDF_0.setLegends(["instrumental marg", "marg"])
graph_CDF_0.setTitle("First component")
view = otv.View(graph_CDF_0)

graph_CDF_1 = marginals[1].drawCDF()
graph_CDF_1.add(distribution.getMarginal(1).drawCDF())
graph_CDF_1.setLegends(["instrumental marg", "marg"])
graph_CDF_1.setTitle("Second component")
view = otv.View(graph_CDF_1)

# %%
# At last, we check that the copula of the multivariate distribution is the specified core
# which was a copula.
cop_dist = distribution.getCopula()
graph_cop = cop_dist.drawPDF()
# Get the Contour Drawable's actual implementation from the Graph
# produced by drawPDF in order to access all its methods
contour_cop = cop.drawPDF().getDrawable(1).getImplementation()
contour_cop.setLineStyle("dashed")
# Remove the colorbar
contour_cop.setColorBarPosition("")
graph_cop.add(contour_cop)
# Add the contour without a colorbargraph_cop.add(cop.drawPDF())
graph_cop.setTitle("Distribution copula and core")
view = otv.View(graph_cop)

# %%
# We can draw the PDF of the bivariate distribution:
graph = distribution.drawPDF()
view = otv.View(graph)

# %%
# We also draw the CDF :
graph = distribution.drawCDF()
view = otv.View(graph)


# %%
# Second case: the core is not a copula
# ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
# In this case, we use a core which is not a copula. Thus, the instrumental marginals
# are not the marginals of the multivariate distribution.
#
# We first create the instrumental marginals of the distribution:
#
# - a :class:`~openturns.Normal` distribution;
# - a :class:`~openturns.Gumbel` distribution.
#
# We use a :class:`~openturns.Dirichlet` as the core.
inst_marginals = [ot.Normal(), ot.Gumbel()]
core_dir = ot.Dirichlet([2.0, 1.5, 2.5])
distribution = ot.JointDistribution(inst_marginals, core_dir)

# %%
# We can check here that the instrumental marginals are not the marginal distributions.
# In the following graphs, we draw the instrumental marginals and the real marginals, obtained with
# the method :meth:`~openturns.Distribution.getMarginal`.
# First, we draw the probability density functions of each component.
graph_PDF_0 = inst_marginals[0].drawPDF()
graph_PDF_0.add(distribution.getMarginal(0).drawPDF())
graph_PDF_0.setLegends(["instrumental marg", "marg"])
graph_PDF_0.setTitle("First component")
view = otv.View(graph_PDF_0)

graph_PDF_1 = inst_marginals[1].drawPDF()
graph_PDF_1.add(distribution.getMarginal(1).drawPDF())
graph_PDF_1.setLegends(["instrumental marg", "marg"])
graph_PDF_1.setTitle("Second component")
view = otv.View(graph_PDF_1)

# %%
# Then, we draw the cumulative distribution functions.
graph_CDF_0 = inst_marginals[0].drawCDF()
graph_CDF_0.add(distribution.getMarginal(0).drawCDF())
graph_CDF_0.setLegends(["instrumental marg", "marg"])
graph_CDF_0.setTitle("First component")
view = otv.View(graph_CDF_0)

graph_CDF_1 = inst_marginals[1].drawCDF()
graph_CDF_1.add(distribution.getMarginal(1).drawCDF())
graph_CDF_1.setLegends(["instrumental marg", "marg"])
graph_CDF_1.setTitle("Second component")
view = otv.View(graph_CDF_1)

# %%
# At last, we check that the copula of the multivariate distribution is not the specified core.
cop_dist = distribution.getCopula()
graph_cop = cop_dist.drawPDF()
cop_dist_draw = graph_cop.getDrawable(1)
levels = cop_dist_draw.getLevels()
graph_core = core_dir.drawPDF()
core_draw = graph_core.getDrawable(0).getImplementation()
core_draw.setColorBarPosition("")
core_draw.setLevels(levels)
core_draw.setLineStyle("dashed")

graph_cop.add(core_draw)
graph_cop.setTitle("Distribution copula and core")
view = otv.View(graph_cop)

# %%
# We can draw the PDF of the bivariate distribution.
graph = distribution.drawPDF()
view = otv.View(graph)

# %%
# We also draw the CDF.
graph = distribution.drawCDF()
view = otv.View(graph)

# %%
# Use some native multivariate models
# -----------------------------------
# Some models in the library are natively multivariate. We present examples of three of them:
#
#  - the :class:`~openturns.Normal` distribution,
#  - the :class:`~openturns.Student` distribution,
#  - the :class:`~openturns.FiniteDiscreteDistribution` distribution.
#
# The Normal distribution
# ^^^^^^^^^^^^^^^^^^^^^^^
#
# The :class:`~openturns.Normal` distribution is natively multivariate.
# Here we define a bivariate standard unit Normal distribution and display
# its PDF.
dim = 2
distribution = ot.Normal(dim)
graph = distribution.drawPDF()
graph.setTitle("Bivariate standard unit Normal PDF")
view = otv.View(graph)


# %%
# The Student distribution
# ^^^^^^^^^^^^^^^^^^^^^^^^
#
# The :class:`~openturns.Student` distribution is natively multivariate. Here we define a Student distribution in dimension 2 and display its PDF :
dim = 2
R = ot.CorrelationMatrix(dim)
R[1, 0] = -0.2
distribution = ot.Student(4, [0.0, 1.0], [1.0, 1.0], R)
graph = distribution.drawPDF()
graph.setTitle("Bivariate Student PDF")
view = otv.View(graph)

# %%
# The FiniteDiscreteDistribution distribution
# ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
#
# We can also define our own distribution with the :class:`~openturns.FiniteDiscreteDistribution` distribution.
# For instance consider the square :math:`[-1,1] \times [-1, 1]` with some
# random points uniformly drawn. For each point the weight chosen is the square
# of the distance to the origin. The :class:`~openturns.FiniteDiscreteDistribution` class normalizes the weights.

# %%
# We first generate random points in the square.
distUniform2 = ot.JointDistribution([ot.Uniform(-1.0, 1.0)] * 2)
N = 100
sample = distUniform2.getSample(N)

# %%
# We then build the points and weights for the `FiniteDiscreteDistribution` distribution.
points = []
weights = []
for i in range(N):
    points.append(sample[i, :])
    weights.append((sample[i, 0] ** 2 + sample[i, 1] ** 2) ** 2)

# %%
# We build the distribution :
distribution = ot.FiniteDiscreteDistribution(points, weights)
graph = distribution.drawPDF()
graph.setTitle("User defined PDF")

# %%
# We can generate and display a sample from this distribution.
omega = distribution.getSample(100)
cloud = ot.Cloud(
    omega, "black", "fdiamond", "Sample from FiniteDiscreteDistribution distribution"
)
graph.add(cloud)
view = otv.View(graph)

# %%
# As expected most values are near the edge of the square where the PDF is the higher.

# %%
# Display all figures
otv.View.ShowAll()



#!/usr/bin/env python

import openturns as ot
from openturns.testing import assert_almost_equal

ot.RandomGenerator.SetSeed(1)

distributionR = ot.LogNormalMuSigma(300.0, 30.0, 0.0).getDistribution()
distributionF = ot.Normal(75.0e3, 5.0e3)
marginals = [distributionR, distributionF]
distribution = ot.JointDistribution(marginals)

model = ot.SymbolicFunction(["R", "F"], ["R - F / (pi_ * 100.0)"])
vect = ot.RandomVector(distribution)
g = ot.CompositeRandomVector(model, vect)
event = ot.ThresholdEvent(g, ot.Less(), -50.0)

distributionMargin1 = ot.LogNormalMuSigma().getDistribution()
distributionMargin2 = ot.Normal()
auxMarginals = [distributionMargin1, distributionMargin2]
auxDistribution = ot.JointDistribution(auxMarginals)

activeParameters = [0, 1, 2, 3, 4]
bounds = ot.Interval([3, 0.09, 0.0, 50e3, 2e3], [7, 0.5, 0.5, 100e3, 10e3])
initialTheta = [5.70, 0.1, 0.0, 75.0e3, 5.0e3]

algo = ot.PhysicalSpaceCrossEntropyImportanceSampling(
    event, auxDistribution, activeParameters, initialTheta, bounds, 0.3
)

algo.setKeepSample(True)
algo.setOptimizationAlgorithm(ot.TNC())
algo.run()
result = algo.getResult()
print(result)
assert_almost_equal(result.getProbabilityEstimate(), 0.000135442, 5e-2, 0.0)

# check that the event sample is right
stepsNumber = algo.getStepsNumber()
inputEventSample = algo.getInputSample(stepsNumber - 1, algo.EVENT1)
outputEventSample = algo.getOutputSample(stepsNumber - 1, algo.EVENT1)
outputG = model(inputEventSample)
diffSample = outputG - outputEventSample
assert_almost_equal(diffSample.computeMean(), [0.0])



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()


size = 5
dim = 6
refSample = ot.Sample(size, dim)
for i in range(size):
    p = ot.Point(dim)
    for j in range(dim):
        p[j] = i + j
    refSample[i] = p
print("ref. sample=", repr(refSample))
experiment = ot.BootstrapExperiment(refSample)
print("experiment = ", experiment)
sample, weights = experiment.generateWithWeights()
print("sample = ", repr(sample))
print("weights = ", repr(weights))
print("selection=", ot.BootstrapExperiment.GenerateSelection(size, size))
